/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003,2004 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package javax.xml.bind;

import java.io.PrintStream;
import java.io.PrintWriter;

/** <p>This is the main exception class of JAXB. All other
 * exception classes (except the {@link javax.xml.bind.TypeConstraintException},
 * which is a {@link java.lang.RuntimeException} are derived from the
 * <code>JAXBException</code>.</p>
 *
 * @author JSR-31
 * @since JAXB1.0
 */
public class JAXBException extends Exception {
    private String errorCode;
    private Throwable linkedException;
    
    /** <p>Creates a new <code>JAXBException</code> with the specified
     * detail message.</p>
     * @param pMessage The detail message.
     */
    public JAXBException(String pMessage) {
        super(pMessage);
    }
    
    /** <p>Creates a new <code>JAXBException</code> with the specified
     * detail message and vendor specific error code.</p>
     * @param pMessage The detail message.
     * @param pErrorCode The error code.
     */
    public JAXBException(String pMessage, String pErrorCode) {
        super(pMessage);
        errorCode = pErrorCode;
    }
    
    /** <p>Creates a new <code>JAXBException</code> with the specified
     * linked exception.</p>
     * @param pLinkedException The linked exception.
     */
    public JAXBException(Throwable pLinkedException) {
        linkedException = pLinkedException;
    }
    
    /** <p>Creates a new <code>JAXBException</code> with the specified
     * detail message and linked exception.</p>
     * @param pMessage The detail message.
     * @param pLinkedException The linked exception.
     */
    public JAXBException(String pMessage, Throwable pLinkedException) {
        super(pMessage);
        linkedException = pLinkedException;
    }
    
    /** <p>Creates a new <code>JAXBException</code> with the specified
     * detail message, error code, and linked exception.</p>
     * @param pMessage The detail message.
     * @param pErrorCode The vendor specific error code.
     * @param pLinkedException The linked exception.
     */
    public JAXBException(String pMessage, String pErrorCode,
						 Throwable pLinkedException) {
        super(pMessage);
        errorCode = pErrorCode;
        linkedException = pLinkedException;
    }
    
    /** <p>Returns the vendor specific error code, if any, or null.</p>
     */
    public String getErrorCode() {
        return errorCode;
    }
    
    /** <p>Returns the linked exception, if any, or null.</p>
     */
    public Throwable getLinkedException() {
        return linkedException;
    }
    
    /** <p>Sets the linked exception.</p>
     * @param pLinkedException The linked exception or null.
     */
    public void setLinkedException(Throwable pLinkedException) {
        linkedException = pLinkedException;
    }
    
    /** <p>Converts the linked exception into a String. Overridden,
     * because the returned string should contain the vendor specific
     * error code, if any.</p>
     */
    public String toString() {
        if (errorCode == null  ||  errorCode.length() == 0) {
            return super.toString();
        } else {
            return errorCode + ": " + super.toString();
        }
    }
    
    public void printStackTrace(PrintStream pStream) {
        super.printStackTrace(pStream);
        Throwable t = getLinkedException();
        if (t != null) {
            pStream.println("Caused by:");
            t.printStackTrace(pStream);
        }
    }

    public void printStackTrace(PrintWriter pWriter) {
        super.printStackTrace(pWriter);
        Throwable t = getLinkedException();
        if (t != null) {
            pWriter.println("Caused by:");
            t.printStackTrace(pWriter);
        }
    }
}
