/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package javax.xml.bind.helpers;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.bind.ValidationEventLocator;

import org.w3c.dom.Node;
import org.xml.sax.Locator;
import org.xml.sax.SAXParseException;


/** <p>Default implementation of a {@link javax.xml.bind.ValidationEventLocator}.</p>
 *
 * @author JSR-31
 * @since JAXB1.0
 */
public class ValidationEventLocatorImpl implements ValidationEventLocator {
  private URL url;
  private int offset, lineNumber, columnNumber;
  private Object object;
  private Node node;

  /** <p>Creates a new instance of <code>ValidationEventLocatorImpl</code>.</p>
   */
  public ValidationEventLocatorImpl() {
    offset = lineNumber = columnNumber = -1;
  }

  /** <p>Creates a new instance of <code>ValidationEventLocatorImpl</code>
   * by copying data from the given {@link org.xml.sax.Locator}.</p>
   * @param pLocator The SAX locator where to copy from.
   */
  public ValidationEventLocatorImpl(Locator pLocator) {
     if (pLocator == null) {
       lineNumber = columnNumber = -1;
     } else {
       columnNumber = pLocator.getColumnNumber();
       lineNumber = pLocator.getLineNumber();
       String u = pLocator.getSystemId();
       if (u != null) {
         try {
           url = new URL(pLocator.getSystemId());
         } catch (MalformedURLException e) {
         }
       }
     }
     offset = -1;
  }

  /** <p>Creates a new instance of <code>ValidationEventLocatorImpl</code>
   * by setting the node property.</p>
   * @param pNode The node being referenced.
   */
  public ValidationEventLocatorImpl(Node pNode) {
    node = pNode;
    offset = lineNumber = columnNumber = -1;
  }

  /** <p>Creates a new instance of <code>ValidationEventLocatorImpl</code>
   * by setting the object property.</p>
   * @param pObject The object being referenced.
   */
  public ValidationEventLocatorImpl(Object pObject) {
    object = pObject;
    offset = lineNumber = columnNumber = -1;
  }

  /** <p>Creates a new instance of <code>ValidationEventLocatorImpl</code>
   * by copying data from the given {@link org.xml.sax.SAXParseException}.</p>
   * @param pException The SAX exception where to copy from.
   */
  public ValidationEventLocatorImpl(SAXParseException pException) {
     columnNumber = pException.getColumnNumber();
     lineNumber = pException.getLineNumber();
     String u = pException.getSystemId();
     if (u != null) {
       try {
         url = new URL(pException.getSystemId());
       } catch (MalformedURLException e) {
       }
     }
     offset = -1;
  }

  /** <p>Sets the URL.</p>
   */
  public void setURL(URL pURL) {
    url = pURL;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getURL()
   */
  public URL getURL() {
    return url;
  }

  /** <p>Sets the offset.</p>
   */
  public void setOffset(int pOffset) {
    offset = pOffset;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getOffset()
   */
  public int getOffset() {
    return offset;
  }

  /** <p>Sets the line number.</p>
   */
  public void setLineNumber(int pLineNumber) {
    lineNumber = pLineNumber;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getLineNumber()
   */
  public int getLineNumber() {
    return lineNumber;
  }

  /** <p>Sets the column number.</p>
   */
  public void setColumnNumber(int pColumnNumber) {
    columnNumber = pColumnNumber;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getColumnNumber()
   */
  public int getColumnNumber() {
    return columnNumber;
  }

  /** <p>Sets the object.</p>
   */
  public void setObject(Object pObject) {
    object = pObject;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getObject()
   */
  public Object getObject() {
    return object;
  }

  /** <p>Sets the node.</p>
   */
  public void setNode(Node pNode) {
    node = pNode;
  }

  /* @see javax.xml.bind.ValidationEventLocator#getNode()
   */
  public Node getNode() {
    return node;
  }
}
