/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme;

import javax.xml.bind.Element;
import javax.xml.bind.JAXBException;


/** <p>The <code>persistence manager</code> (or <code>PM</code>
 * for short) is responsible for reading objects from the
 * database or storing them into the database.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface PM {
  /** <p>Initializes the PM. Called from the
   * {@link org.apache.ws.jaxme.impl.JAXBContextImpl} upon initialization.</p>
   * @param pManager The manager being queried for configuration details.
   */
  public void init(JMManager pManager) throws JAXBException;

  /** <p>Returns the manager being queried for configuration details.</p>
   */
  public JMManager getManager();

  /** <p>Reads documents matching the given query. For any document
   * matching, the Observer's notify method is executed with the
   * matching document as an argument.</p>
   *
   * @param pObserver This Observer is notified for any matching document.
   *   The document is added as an argument.
   * @param pQuery The query to perform.
   */
  public void select(Observer pObserver, String pQuery) throws PMException;

  /** <p>Reads documents matching the given query. For any document
   * matching, the Observer's notify method is executed with the
   * matching document as an argument.</p>
   * <p>The query may contain placeholders. If it does, you have
   * to supply an instance of {@link PMParams} with the placeholder
   * values. Example:
   * <pre>
   *   manager.select("Name = ? and Id = ?",
   *                  new PMParams().addString("Someone").addInt(4));
   * </pre></p>
   *
   * @param pObserver This Observer is notified for any matching document.
   *   The document is added as an argument.
   * @param pQuery The query to perform. May contain placeholders.
   * @param pPlaceHolderArgs An array of objects or null, if the
   *   query doesn't contain any placeholders.
   */
  public void select(Observer pObserver, String pQuery,
                      PMParams pPlaceHolderArgs) throws PMException;

  /** <p>Returns an iterator to all documents matching the given query.</p>
   *
   * @param pQuery The query to perform.
   */
  public java.util.Iterator select(String pQuery) throws PMException;

  /** <p>Returns an iterator to all documents matching the given query.
   * The query may contain placeholders. If it does, you have
   * to supply an instance of {@link PMParams} with the placeholder
   * values. Example:
   * <pre>
   *   manager.select("Name = ? and Id = ?",
   *                  new PMParams().addString("Someone").addInt(4));
   * </pre></p>
   *
   * @param pQuery The query to perform. May contain placeholders.
   * @param pPlaceHolderArgs An array of objects or null, if the
   *   query doesn't contain any placeholders.
   */
  public java.util.Iterator select(String pQuery,
                                    PMParams pPlaceHolderArgs)
    throws PMException;

  /** <p>Inserts the given document into the database.</p>
   */
  public void insert(Element element) throws PMException;

  /** <p>Updates the given document in the database.</p>
   */
  public void update(Element element) throws PMException;

  /** <p>Deletes the given document from the database.</p>
   */
  public void delete(Element element) throws PMException;

  /** <p>Creates a new, empty element.</p>
   */
  public Object create() throws JAXBException;
}
