/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003,2004 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg;

import org.apache.ws.jaxme.js.JavaSource;
import org.apache.ws.jaxme.xs.XSAttribute;
import org.apache.ws.jaxme.xs.XSWildcard;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/** <p>Interface of a source generator for complex types.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface ComplexTypeSG {
  /** <p>Initializes the instance.</p>
   */
  public void init() throws SAXException;

  /** <p>Returns the {@link TypeSG} that created this instance.</p>
   */
  public TypeSG getTypeSG();

  /** <p>Returns the types {@link Locator}.</p>
   */
  public Locator getLocator();

  /** <p>Returns the datatypes runtime type, which is the binding
   * interface.</p>
   */
  public Context getClassContext();

  /** <p>Generates the types interface as a standalone class.
   * This is used for global types.</p>
   */
  public JavaSource getXMLInterface() throws SAXException;

  /** <p>Generates the types interface as an inner class of the given.</p>
   */
  public JavaSource getXMLInterface(JavaSource pSource) throws SAXException;

  /** <p>Generates the types interface by adding methods
   * to the given class.</p>
   */
  public void generateXMLInterfaceMethods(JavaSource pSource) throws SAXException;

  /** <p>Generates the types implementation as a standalone class.
   * This is used for global types.</p>
   */
  public JavaSource getXMLImplementation() throws SAXException;

  /** <p>Generates the types implementation as an inner class of the given.</p>
   */
  public JavaSource getXMLImplementation(JavaSource pSource) throws SAXException;

  /** <p>Generates the types XML serializer as a standalone class.
   * This is used for global types.</p>
   */
  public JavaSource getXMLSerializer() throws SAXException;

  /** <p>Generates the types XML serializer as an inner class of the given.
   * This is used for local types.</p>
   */
  public JavaSource getXMLSerializer(JavaSource pSource) throws SAXException;

  /** <p>Generates the types XML handler as a standalone class.
   * This is used for global types.</p>
   */
  public JavaSource getXMLHandler() throws SAXException;

  /** <p>Generates the types XML handler as an inner class of the given.
   * This is used for local types.</p>
   */
  public JavaSource getXMLHandler(JavaSource pSource) throws SAXException;

  /** <p>Generates the types implementation by adding methods
   * to the given class.</p>
   */
  public void generateXMLImplementationMethods(JavaSource pSource) throws SAXException;

  /** <p>Generates the types serializer by adding methods to the
   * given class.</p>
   */
  public void generateXMLSerializerMethods(JavaSource pSource) throws SAXException;

  /** <p>Generates the types handler by adding methods to the
   * given class.</p>
   */
  public void generateXMLHandlerMethods(JavaSource pSource) throws SAXException;

  /** <p>Returns whether the data type has attributes.</p>
   */
  public boolean hasAttributes();

  /** <p>Creates a new instance of
   * {@link org.apache.ws.jaxme.generator.sg.AttributeSGChain}
   * generating the given attribute.</p>
   */
  public Object newAttributeSG(XSAttribute pAttribute) throws SAXException;

  /** <p>Creates a new instance of
   * {@link org.apache.ws.jaxme.generator.sg.AttributeSGChain}
   * generating the given wildcard attributes.</p>
   */
  public Object newAttributeSG(XSWildcard pWildcard) throws SAXException;

  /** <p>Adds the given {@link AttributeSG} to the list of attributes
   * and invokes the method {@link org.apache.ws.jaxme.generator.sg.SGItem#init()}
   * on it.</p>
   */
  public void addAttributeSG(AttributeSG pAttribute) throws SAXException;

  /** <p>Returns the data types array of attributes.</p>
   */
  public AttributeSG[] getAttributes();

  /** <p>Returns whether the data type has simple content.</p>
   */
  public boolean hasSimpleContent();

  /** <p>If the complex type has simple content: Creates an instance of
   * {@link org.apache.ws.jaxme.generator.sg.SimpleContentSGChain} generating the type.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.SimpleContentSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.SimpleContentSGChain}
   * interface.</p>
   */
  public Object newSimpleContentTypeSG() throws SAXException;

  /** <p>If the data type has simple content: Returns an instance of
   * {@link SimpleContentSG}. Uses {@link #newSimpleContentTypeSG()}
   * internally.</p>
   */
  public SimpleContentSG getSimpleContentSG();

  /** <p>If the complex type has complex content: Creates an instance of
   * {@link org.apache.ws.jaxme.generator.sg.ComplexContentSGChain} generating the given complex type.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.ComplexContentSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.ComplexContentSGChain}
   * interface.</p>
   */
  public Object newComplexContentTypeSG() throws SAXException;

  /** <p>If the data type has complex content: Returns an instance of
   * {@link ComplexContentSG}. Uses {@link #newComplexContentTypeSG()}
   * internally.</p>
   */
  public ComplexContentSG getComplexContentSG();
}
