/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg;

import java.io.Serializable;

/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface Facet {
  public class Type implements Serializable {
    private final String name;
    Type(String pName) { name = pName; }
    public String getName() { return name; }
    public String toString() { return name; }
    private static final Type[] instances = new Type[]{
      ENUMERATION, FRACTION_DIGITS, LENGTH, MAX_EXCLUSIVE, MAX_INCLUSIVE,
      MAX_LENGTH, MIN_EXCLUSIVE, MIN_INCLUSIVE, MIN_LENGTH, PATTERN, TOTAL_DIGITS 
    };
    public static Type valueOf(String pName) {
      for (int i = 0;  i < instances.length;  i++) {
        if (instances[i].name.equals(pName)) {
          return instances[i];
        }
      }
      throw new IllegalArgumentException("Unknown facet type: " + pName);
    }
    public int hashCode() { return name.hashCode(); }
    public boolean equals(Object o) {
      return o != null  &&  (o instanceof Type)  &&  ((Type) o).name.equals(name);
    }
  };

  /** <p>The facet type enumeration. Use the method {@link #getValues}
   * to query for the values.</p>
   */
  public static final Type ENUMERATION = new Type("enumeration");

  /** <p>The facet type fractionDigits. Use the method {@link #getNumValue}
   * to query for the values.</p>
   */
  public static final Type FRACTION_DIGITS = new Type("fractionDigits");

  /** <p>The facet type length. Use the method {@link #getNumValue}
   * to query for the values.</p>
   */
  public static final Type LENGTH = new Type("length");

  /** <p>The facet type maxLength. Use the method {@link #getNumValue}
   * to query for the values.</p>
   */
  public static final Type MAX_LENGTH = new Type("maxLength");

  /** <p>The facet type maxExclusive. Use the method {@link #getValue} to
   * query for the value.</p>
   */
  public static final Type MAX_EXCLUSIVE = new Type("maxExclusive");

  /** <p>The facet type maxInclusive. Use the method {@link #getValue} to
   * query for the value.</p>
   */
  public static final Type MAX_INCLUSIVE = new Type("maxInclusive");

  /** <p>The facet type minLength. Use the method {@link #getNumValue}
   * to query for the values.</p>
   */
  public static final Type MIN_LENGTH = new Type("minLength");

  /** <p>The facet type minExclusive. Use the method {@link #getValue} to
   * query for the value.</p>
   */
  public static final Type MIN_EXCLUSIVE = new Type("minExclusive");

  /** <p>The facet type minInclusive. Use the method {@link #getValue} to
   * query for the value.</p>
   */
  public static final Type MIN_INCLUSIVE = new Type("minInclusive");

  /** <p>The facet type pattern. Use the method {@link #getValues}
   * to query for the values.</p>
   */
  public static final Type PATTERN = new Type("pattern");

  /** <p>The facet type totalDigits. Use the method {@link #getNumValue}
   * to query for the values.</p>
   */
  public static final Type TOTAL_DIGITS = new Type("totalDigits");

  /** <p>Returns the facet type.</p>
   */
  public Type getType();

  /** <p>If the facet has the types {@link #ENUMERATION} or {@link #PATTERN}:
   * Used to fetch the possible values.</p>
   */
  public String[] getValues();

  /** <p>If the facet has the types {@link #MAX_EXCLUSIVE}, {@link #MIN_EXCLUSIVE},
   * {@link #MAX_INCLUSIVE}, or {@link #MIN_INCLUSIVE}: Returns the facet value.</p>
   */
  public String getValue();

  /** <p>If the facet has the types {@link #FRACTION_DIGITS} or {@link #TOTAL_DIGITS}:
   * Returns the facet value.</p>
   */
  public long getNumValue();
}
