/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg;

import org.apache.ws.jaxme.generator.Generator;
import org.apache.ws.jaxme.xs.XSAny;
import org.apache.ws.jaxme.xs.XSElement;
import org.apache.ws.jaxme.xs.XSEnumeration;
import org.apache.ws.jaxme.xs.XSGroup;
import org.apache.ws.jaxme.xs.XSObjectFactory;
import org.apache.ws.jaxme.xs.XSParser;
import org.apache.ws.jaxme.xs.XSSchema;
import org.apache.ws.jaxme.xs.XSType;
import org.apache.ws.jaxme.xs.xml.XsObjectFactory;
import org.apache.ws.jaxme.xs.xml.XsQName;
import org.xml.sax.SAXException;


/** <p>Interface of a factory for SG classes.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface SGFactory {
  /** <p>Returns the {@link org.apache.ws.jaxme.generator.Generator}.</p>
   */
  public Generator getGenerator();

  /** <p>Initializes the {@link SGFactory}.</p>
   */
  public void init();

  /** <p>Creates a new instance of {@link org.apache.ws.jaxme.xs.xml.XsObjectFactory},
   * used by the parser.</p>
   */
  public XsObjectFactory newXsObjectFactory() throws SAXException;

  /** <p>Creates a new instance of {@link org.apache.ws.jaxme.xs.XSObjectFactory},
   * used by the parser.</p>
   */
  public XSObjectFactory newXSObjectFactory() throws SAXException;

  /** <p>Creates a new instance of {@link SchemaSG} generating the
   * given schema.</p>
   */
  public Object newSchemaSG(XSSchema pSchema) throws SAXException;
  /** <p>Returns an instance of {@link SchemaSG} generating the
   * given schema.</p>
   */
  public SchemaSG getSchemaSG(XSSchema pSchema) throws SAXException;
  /** <p>Returns the factorys instance of {@link SchemaSG}.</p>
   * @throws IllegalStateException The factory has not yet created an
   *    instance of SchemaSG. In other words, the methods
   *    {@link #newSchemaSG(XSSchema)} and {@link #getSchemaSG(XSSchema)}
   *    have not yet been invoked.
   */
  public SchemaSG getSchemaSG();

  /** <p>Creates a new instance of {@link ObjectSG} generating the
   * given element.</p>
   */
  public Object newObjectSG(XSElement pElement) throws SAXException;

  /** <p>Returns an instance of {@link ObjectSG} generating the
   * given element.</p>
   */
  public ObjectSG getObjectSG(XSElement pElement) throws SAXException;

  /** <p>Creates a new instance of {@link ObjectSG} generating the
   * given element in the given context.</p>
   */
  public Object newObjectSG(XSElement pElement, Context pContext) throws SAXException;

  /** <p>Returns an instance of {@link ObjectSG} generating the given
   * element in the given context.</p>
   */
  public ObjectSG getObjectSG(XSElement pElement, Context pContext) throws SAXException;

  /** <p>Creates a new instance of {@link ObjectSG} generating the given
   * wildcard in the given context.</p>
   */
  public Object newObjectSG(XSAny any);

  /** <p>Returns an instance of {@link ObjectSG} generating the given
   * wildcard in the given context.</p>
   */
  public ObjectSG getObjectSG(XSAny pWildcard, Context pContext) throws SAXException;

  /** <p>Creates a new instance of {@link GroupSG} generating the
   * given schema.</p>
   */
  public GroupSG newGroupSG(XSGroup pGroup) throws SAXException;

  /** <p>Returns an instance of {@link GroupSG} generating the
   * given schema.</p>
   */
  public GroupSG getGroupSG(XSGroup pGroup) throws SAXException;

  /** <p>Creates a new instance of {@link GroupSG}, which is embedded
   * into the given {@link Context}.</p>
   */
  public GroupSG newGroupSG(XSGroup pGroup, Context pClassContext) throws SAXException;

  /** <p>Returns an instance of {@link GroupSG}, which is embedded
   * into the given {@link Context}.</p>
   */
  public GroupSG getGroupSG(XSGroup pGroup, Context pClassContext) throws SAXException;

  /** <p>Creates a new, global instance of
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * generating the given type.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * interface.</p>
   */
  public Object newTypeSG(XSType pType) throws SAXException;

  /** <p>Returns a global instance of {@link TypeSG} generating
   * the given type.</p>
   */
  public TypeSG getTypeSG(XSType pType) throws SAXException;

  /** <p>Creates a new, local instance of
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain},
   * generating the given type within the given {@link Context}.
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * interface.</p>
   */
  public Object newTypeSG(XSType pType, Context pClassContext, XsQName pName) throws SAXException;

  /** <p>Returns a local instance of {@link TypeSG} generating
   * the given type within the given {@link Context}.</p>
   */
  public TypeSG getTypeSG(XSType pType, Context pClassContext, XsQName pName) throws SAXException;

  /** <p>Creates a new, local instance of
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain},
   * as if it were a global type with the given name.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.TypeSGChain}
   * interface.</p>
   */
  public Object newTypeSG(XSType pType, XsQName pName) throws SAXException;

  /** <p>Returns a new instance of {@link TypeSG} generating
   * the given type, as if it were a global type with the
   * given name.</p>
   */
  public TypeSG getTypeSG(XSType pType, XsQName pName) throws SAXException;

  /** <p>Creates a new instance of {@link XSParser}.</p>
   */
  public XSParser newXSParser() throws SAXException;

  /** <p>Creates a new enumeration facet.</p>
   */
  public Facet newFacet(XSType pType, XSEnumeration[] pEnumerations) throws SAXException;

  /** <p>Returns an array of all groups created by the factory.</p>
   */
  public GroupSG[] getGroups();

  /** <p>Returns an array of all types created by the factory.</p>
   */
  public TypeSG[] getTypes();

  /** <p>Returns an array of all objects created by the factory.</p>
   */
  public ObjectSG[] getObjects();
}
