/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg;

import org.apache.ws.jaxme.js.DirectAccessible;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.js.JavaSource;
import org.xml.sax.SAXException;


/** <p>Interface of a source generator for complex types.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface SimpleTypeSG extends SGItem {
  /** <p>Returns whether the simple type is atomic.</p>
   */
  public boolean isAtomic();

  /** <p>Returns whether the simple type is a list.</p>
   */
  public boolean isList();

  /** <p>Returns whether the simple type is a union.</p>
   */
  public boolean isUnion();

  /** <p>Returns whether the simple type is nullable.</p>
   */
  public boolean isNullable();

  /** <p>Sets whether the simple type is nullable.</p>
   */
  public void setNullable(boolean pNullable);

  /** <p>If the simple type is atomic: Returns its atomic type details.</p>
   */
  public AtomicTypeSG getAtomicType();

  /** <p>If the simple type is a list: Returns its item type details.</p>
   * @throws IllegalStateException The type is no list.
   */
  public ListTypeSG getListType();

  /** <p>If the simple type is a union: Returns its union type details.</p>
   * @throws IllegalStateException The type is no union.
   */
  public UnionTypeSG getUnionType();

  /** <p>Returns the data types runtime type.</p>
   */
  public JavaQName getRuntimeType();

  /** <p>Returns a piece of Java code converting the string <code>pValue</code>
   * into the runtime type. Conversion occurs at runtime, using the
   * given instance of {@link org.apache.ws.jaxme.JMHandler.Data}.</p>
   * @param pMethod The method performing the type convertion.
   * @param pValue The value being casted
   * @param pData A piece of Java code holding an instance of
   * {@link org.apache.ws.jaxme.JMHandler.Data}; may be used to support the conversion.
   */
  public Object getCastFromString(JavaMethod pMethod, Object pValue, Object pData) throws SAXException;

  /** <p>Returns a piece of Java code converting the runtime type
   * <code>pValue</code> into a string. Conversion occurs at runtime, using the
   * given instance of {@link org.apache.ws.jaxme.JMXmlSerializer.Data}.</p>
   */
  public Object getCastToString(JavaMethod pMethod, Object pValue, DirectAccessible pData) throws SAXException;

  /** <p>Returns a piece of Java code converting the string <code>pValue</code>
   * into the runtime type. Conversion occurs at compile time.</p>
   */
  public Object getCastFromString(String pValue) throws SAXException;

  /** <p>Returns whether the simple type does have a "set" method.</p>
   */
  public boolean hasSetMethod() throws SAXException;

  /** <p>Returns the collection type, which is either of "indexed" (an array)
   * or a list implementation, as specified by JAXB's property tag.</p>
   */
  public String getCollectionType();

  /** <p>Returns all of the simple types facets.</p>
   */
  public Facet[] getFacets();

  /** <p>Returns the simple types facets with the given type or null, if no
   * such facet exists.</p>
   */
  public Facet getFacet(Facet.Type pType);

  /** <p>Invokes the given {@link SGlet} on any value, assuming they
   * are non null.</p>
   */
  public void forAllValues(JavaMethod pMethod, Object pValue, SGlet pSGlet) throws SAXException;

  /** <p>Invokes the given {@link SGlet} on any non null value.</p>
   */
  public void forAllNonNullValues(JavaMethod pMethod, Object pValue, SGlet pSGlet) throws SAXException;

  /** <p>Generates helper classes required by the simple type.</p>
   */
  public void generate() throws SAXException;

  /** <p>Generates helper classes required by the simple type.
   * The generated classes are inner classes of the given.</p>
   */
  public void generate(JavaSource pSource) throws SAXException;

  /** <p>Returns code creating a boolean value indicating whether the given values
   * are equal.</p>
   */
  public Object getEqualsCheck(JavaMethod pMethod, Object pValue1, Object pValue2) throws SAXException;

  /** <p>Returns the types initial value, as created by the constructor.</p>
   */
  public Object getInitialValue(JavaSource pSource) throws SAXException;

  /** <p>Generates a set method for the simple type.</p>
   */
  public JavaMethod getXMLSetMethod(JavaSource pSource,
                                     String pFieldName, String pMethodName) throws SAXException;

  /** <p>Adds code for validating the value <code>pValue</code> to the "add" or
   * "set" method <code>pMethod</code>.</p>
   */
  public void addValidation(JavaMethod pMethod, DirectAccessible pValue) throws SAXException;
}
