/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003,2004 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg;

import org.apache.ws.jaxme.js.JavaField;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.js.JavaSource;
import org.apache.ws.jaxme.xs.xml.XsQName;
import org.apache.ws.jaxme.xs.xml.XsSchemaHeader;
import org.xml.sax.SAXException;


/** <p>Interface of a source generator for types; applies both to
 * simple and complex types.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface TypeSG extends SGItem {
  /** <p>A property allows to store custom data related to the type.
   * Properties are used by external source generators. For example,
   * the JDBC source generator will use this to store the complex
   * types table name here.</p>
   */
  public void setProperty(String pName, Object pValue);

  /** <p>A property allows to store custom data related to the type.
   * Properties are used by external source generators. For example,
   * the JDBC source generator will use this to store the complex
   * types table name here.</p>
   */
  public Object getProperty(String pName);

  /** <p>Returns whether this is a global type.</p>
   */
  public boolean isGlobalType();

  /** <p>Returns whether this type is generated with a global class.</p>
   */
  public boolean isGlobalClass();

  /** <p>If the type is global: Returns the types name.</p>
   * @throws IllegalStateException The type isn't global.
   */
  public XsQName getName();

  /** <p>Returns whether the type is complex. If so, it is valid to
   * invoke the method {@link #getComplexTypeSG}. Otherwise, you may
   * invoke the method {@link #getSimpleTypeSG}.</p>
   */
  public boolean isComplex();

  /** <p>If the type is simple: Creates a new instance of
   * {@link org.apache.ws.jaxme.generator.sg.SimpleTypeSGChain} generating the type.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.SimpleTypeSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.SimpleTypeSGChain}
   * interface.</p>
   */
  public Object newSimpleTypeSG() throws SAXException;

  /** <p>If the type is complex: Creates an instance of
   * {@link org.apache.ws.jaxme.generator.sg.ComplexTypeSGChain} generating the type.</p>
   * <p><em>Implementation note</em>: The type
   * {@link org.apache.ws.jaxme.generator.sg.ComplexTypeSGChain}
   * must not be exposed in the interface, because the interface
   * class is used to generate this type. In other words, this
   * interface must be compilable without the
   * {@link org.apache.ws.jaxme.generator.sg.ComplexTypeSGChain}
   * interface.</p>
   */
  public Object newComplexTypeSG() throws SAXException;

  /** <p>If the type is complex: Returns an instance of
   * {@link ComplexTypeSG} generating the type.</p>
   */
  public ComplexTypeSG getComplexTypeSG();

  /** <p>If the type is simple: Returns an instance of {@link SimpleTypeSG}
   * for generating the type.</p>
   *
   * @throws IllegalStateException The type is complex.
   */
  public SimpleTypeSG getSimpleTypeSG();

  /** <p>Generates a Java field for an instance of this type.</p>
   */
  public JavaField getXMLField(JavaSource pSource, 
                                     String pFieldName, String pDefaultValue) throws SAXException;

  /** <p>Generates a get method returning an instance of this type.</p>
   */
  public JavaMethod getXMLGetMethod(JavaSource pSource,
                                     String pFieldName, String pMethodName) throws SAXException;

  /** <p>Generates a set method returning an instance of this type.</p>
   */
  public JavaMethod getXMLSetMethod(JavaSource pSource,
                                     String pFieldName, String pMethodName) throws SAXException;

  /** <p>Generates an "isSet" method returning whether the field is set.</p>
   */
  public JavaMethod getXMLIsSetMethod(JavaSource pSource,
                                       String pFieldName, String pMethodName) throws SAXException;

  /** <p>Returns the types runtime type.</p>
   */
  public JavaQName getRuntimeType() throws SAXException;

  /** <p>Generates the types sources.</p>
   */
  public void generate() throws SAXException;

  /** <p>Generates the types sources as an inner class of the given.</p>
   */
  public void generate(JavaSource pSource) throws SAXException;

  /** <p>Returns whether the type is a restriction of another type.</p>
   */
  public boolean isRestriction();

  /** <p>If the type is a restriction: Returns the restricted type.</p>
   * @throws IllegalStateException The type is no restriction.
   */
  public TypeSG getRestrictedType();

  /** <p>Returns whether the type is an extension of another type.</p>
   */
  public boolean isExtension();

  /** <p>If the type is an extension: Returns the extended type.</p>
  * @throws IllegalStateException The type is no extension.
   */
  public TypeSG getExtendedType();

  /** <p>Returns information on the types syntactical context.</p>
   */
  public XsSchemaHeader getSchemaHeader();
}
