/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.ws.jaxme.generator.sg.Context;
import org.apache.ws.jaxme.generator.sg.SchemaSG;
import org.apache.ws.jaxme.generator.util.JavaNamer;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.apache.ws.jaxme.xs.xml.XsQName;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public abstract class AbstractContext implements Context {
  private XsQName name;
  private JavaQName xmlInterfaceName, xmlImplementationName, xmlHandlerName, xmlSerializerName;
  private JavaQName xmlValidatorName, pmName;

  public XsQName getName() { return name; }
  protected void setName(XsQName pName) { name = pName; }
  public JavaQName getXMLHandlerName() { return xmlHandlerName; }
  protected void setXMLHandlerName(JavaQName pXmlHandlerName) { xmlHandlerName = pXmlHandlerName; }
  public JavaQName getXMLImplementationName() { return xmlImplementationName; }
  protected void setXMLImplementationName(JavaQName pXmlImplementationName) { xmlImplementationName = pXmlImplementationName; }
  public JavaQName getXMLInterfaceName() { return xmlInterfaceName; }
  protected void setXMLInterfaceName(JavaQName pXmlInterfaceName) {
    if (pXmlInterfaceName.toString().endsWith("Adress.AddressType")  ||
        pXmlInterfaceName.toString().endsWith("Address$AddressType")) {
      throw new IllegalStateException();    
    }
    xmlInterfaceName = pXmlInterfaceName;
  }
  public JavaQName getXMLSerializerName() { return xmlSerializerName; }
  protected void setXMLSerializerName(JavaQName pXmlSerializerName) { xmlSerializerName = pXmlSerializerName; }
  protected void setXMLValidatorName(JavaQName pXmlValidatorName) { xmlValidatorName = pXmlValidatorName; }
  public JavaQName getXMLValidatorName() { return xmlValidatorName; }
  protected void setPMName(JavaQName pPMName) { pmName = pPMName; }
  public JavaQName getPMName() { return pmName; }

  protected static String getClassNameFromLocalName(Locator pLocator, String pLocalName,
                                                      SchemaSG pSchemaSG) throws SAXException {
    if (pLocalName == null  ||  pLocalName.length() == 0) {
      throw new LocSAXException("Invalid local name: " + pLocalName, pLocator);
    }
    return JavaNamer.convert(pLocalName, pSchemaSG);
  }

  protected static String getPackageNameFromURL(URL pURL) {
    List tokens = new ArrayList();
    for (StringTokenizer st = new StringTokenizer(pURL.getHost(), ".");
         st.hasMoreTokens();  ) {
      tokens.add(st.nextToken());
    }
    Collections.reverse(tokens);
    for (StringTokenizer st = new StringTokenizer(pURL.getPath(), "/");
         st.hasMoreTokens();  ) {
      tokens.add(st.nextToken());
    }
  
    if (tokens.size() == 0) {
      throw new IllegalArgumentException("Could not parse URL " + pURL);
    }
    StringBuffer sb = new StringBuffer();
    for (int i = 0;  i < tokens.size();  i++) {
      if (i > 0) {
        sb.append('.');
      }
      String tok = (String) tokens.get(i);
      if (tok.length() == 0) {
        throw new IllegalArgumentException("Could not parse URL " + pURL);
      }
      for (int j = 0;  j < tok.length();  j++) {
        char c = Character.toLowerCase(tok.charAt(j));
        if ((j == 0  &&  Character.isJavaIdentifierStart(c))  ||
            (j > 0  && Character.isJavaIdentifierPart(c))) {
          sb.append(c);
        } else {
          sb.append('_');
        }
      }
    }
    return sb.toString();
  }
  
  protected static String getPackageNameFromURI(Locator pLocator, String pURI) throws SAXException {
    if (pURI == null  ||  pURI.length() == 0) {
      throw new LocSAXException("Unable to derive package name from an empty namespace URI. Use the schemaBindings to specify a package name.", pLocator);
    }
    try {
      URL url = new java.net.URL(pURI);
      return getPackageNameFromURL(url);
    } catch (MalformedURLException e) {
      throw new LocSAXException("Unable to derive package name from an URI, which is no URL: " + pURI,
                                 pLocator);
    }
  }

  public String toString() {
    return super.toString() + " [" + getName() + "," + getXMLInterfaceName() + "," +
        getXMLImplementationName() + "]";
  }
}
