/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.sg.impl;

import org.apache.ws.jaxme.generator.sg.Context;
import org.apache.ws.jaxme.generator.sg.GroupSG;
import org.apache.ws.jaxme.generator.sg.ObjectSG;
import org.apache.ws.jaxme.generator.sg.ParticleSG;
import org.apache.ws.jaxme.generator.sg.ParticleSGChain;
import org.apache.ws.jaxme.generator.sg.PropertySG;
import org.apache.ws.jaxme.generator.sg.PropertySGChain;
import org.apache.ws.jaxme.generator.sg.SGlet;
import org.apache.ws.jaxme.generator.sg.TypeSG;
import org.apache.ws.jaxme.js.DirectAccessible;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.xs.XSAny;
import org.apache.ws.jaxme.xs.XSElement;
import org.apache.ws.jaxme.xs.XSParticle;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class JAXBParticleSG implements ParticleSGChain {
    private final int minOccurs, maxOccurs;
    private final XSParticle.Type type;
    private PropertySG propertySG;
    private final GroupSG groupSG;
    private final ObjectSG objectSG;
    private final Locator locator;
    private XSElement element;
    private XSAny wildcard;

    /** <p>Creates a new instance of JAXBParticleSG.java.</p>
     */
    public JAXBParticleSG(GroupSG pGroup, XSParticle pParticle, Context pClassContext) throws SAXException {
        minOccurs = pParticle.getMinOccurs();
        maxOccurs = pParticle.getMaxOccurs();
        type = pParticle.getType();
        if (pParticle.isGroup()) {
            groupSG = pGroup.getFactory().getGroupSG(pParticle.getGroup(), pClassContext);
            objectSG = null;
	    } else if (pParticle.isElement()) {
	        objectSG = pGroup.getFactory().getObjectSG(pParticle.getElement(), pClassContext);
	        groupSG = null;
	        element = pParticle.getElement();
	    } else if (pParticle.isWildcard()) {
	        objectSG = pGroup.getFactory().getObjectSG(pParticle.getWildcard(), pClassContext);
	        groupSG = null;
	        wildcard = pParticle.getWildcard();
	    } else {
	        throw new IllegalStateException("Particle is neither group, nor element, or wildcard.");
	    }
        locator = pParticle.getLocator();
    }

    public Object newPropertySGChain(ParticleSG pController) {
        PropertySGChain result;
        if (element != null) {
            result = new JAXBPropertySG(objectSG, element);
            element = null;
        } else if (wildcard != null) {
            result = new AnyElementPropertySG(objectSG, wildcard);
            wildcard = null;
        } else {
            throw new IllegalStateException("A new PropertySGChain cannot be obtained.");
        }
        if (maxOccurs > 1  ||  maxOccurs == -1) {
            result = new MultiplePropertySG(result, objectSG, minOccurs, maxOccurs);
        }
        return result;
    }
    
    public void init(ParticleSG pController) throws SAXException {
        if (element != null  ||  wildcard != null) {
            PropertySGChain chain = (PropertySGChain) pController.newPropertySGChain();
            propertySG = new PropertySGImpl(chain);
            propertySG.init();
        }
    }

    public Locator getLocator(ParticleSG pController) { return locator; }
    public int getMinOccurs(ParticleSG pController) { return minOccurs; }
    public int getMaxOccurs(ParticleSG pController) { return maxOccurs; }
    public boolean isMultiple(ParticleSG pController) { return maxOccurs == -1  ||  maxOccurs > 1; }
    public boolean isGroup(ParticleSG pController) { return type.equals(XSParticle.GROUP); }
    public boolean isElement(ParticleSG pController) { return type.equals(XSParticle.ELEMENT); }
    public boolean isWildcard(ParticleSG pController) { return type.equals(XSParticle.WILDCARD); }

    public PropertySG getPropertySG(ParticleSG pController) {
        if (propertySG == null) {
            throw new IllegalStateException("This particle has no PropertySG.");
        }
        return propertySG;
    }

    public ObjectSG getObjectSG(ParticleSG pController) {
        if (objectSG == null) {
            throw new IllegalStateException("This particle is neither an element nor a wildcard.");
        }
        return objectSG;
    }

    public GroupSG getGroupSG(ParticleSG pController) {
        if (groupSG == null) {
            throw new IllegalStateException("This particle is no group.");
        }
        return groupSG;
    }

    public void forAllNonNullValues(ParticleSG pController, JavaMethod pMethod,
									DirectAccessible pElement, SGlet pSGlet) throws SAXException {
        if (pController.isElement()) {
            PropertySG pSG = pController.getPropertySG();
            boolean hasIsSetMethod = pSG.hasIsSetMethod();
            if (hasIsSetMethod) {
                pMethod.addIf(pSG.getXMLIsSetMethodName(), "()");
            }
            TypeSG typeSG = pController.getObjectSG().getTypeSG();
            Object v = pController.getPropertySG().getValue(pElement);
            if (typeSG.isComplex()) {
                pSGlet.generate(pMethod, v);
            } else {
                typeSG.getSimpleTypeSG().forAllValues(pMethod, v, pSGlet);
            }
            if (hasIsSetMethod) {
                pMethod.addEndIf();
            }
        } else {
            // TODO: Implement support for wildcards and subgroups
            throw new IllegalStateException("Wildcards and subgroups are not yet supported.");
        }
    }
}
