/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.types;

import org.apache.ws.jaxme.generator.sg.AtomicTypeSG;
import org.apache.ws.jaxme.generator.sg.SGFactory;
import org.apache.ws.jaxme.generator.sg.SGlet;
import org.apache.ws.jaxme.generator.sg.SchemaSG;
import org.apache.ws.jaxme.generator.sg.SimpleTypeSG;
import org.apache.ws.jaxme.js.DirectAccessible;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.js.JavaQNameImpl;
import org.apache.ws.jaxme.js.JavaSource;
import org.apache.ws.jaxme.js.LocalJavaField;
import org.apache.ws.jaxme.xs.XSType;
import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.xml.sax.SAXException;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class StringSG extends AtomicTypeSGImpl {
  /** <p>Creates a new instance of StringTypeSG.java.</p>
   */
  public StringSG(SGFactory pFactory, SchemaSG pSchema, XSType pType) throws SAXException {
    super(pFactory, pSchema, pType);
  }

  public final static JavaQName STRING_TYPE = JavaQNameImpl.getInstance(String.class);
  protected String getDatatypeName() { return "String"; }

  public JavaQName getRuntimeType(SimpleTypeSG pController) { return STRING_TYPE; }

  public Object getCastFromString(SimpleTypeSG pController, String pValue) {
    return JavaSource.getQuoted(pValue);
  }

  public Object getCastFromString(SimpleTypeSG pController, JavaMethod pMethod, Object pValue, Object pData) {
    return pValue;
  }

  public Object getCastToString(SimpleTypeSG pController, Object pValue, DirectAccessible pData) {
    return pValue;
  }

  public void forAllNonNullValues(SimpleTypeSG pController, JavaMethod pMethod, Object pValue, SGlet pSGlet) throws SAXException {
    LocalJavaField f = pMethod.newJavaField(STRING_TYPE);
    f.addLine(pValue);
    pMethod.addIf(f, " != null");
    pSGlet.generate(pMethod, pValue);
    pMethod.addEndIf();
  }

  public void forAllValues(SimpleTypeSG pController, JavaMethod pMethod, Object pValue, SGlet pSGlet) throws SAXException {
    pSGlet.generate(pMethod, pValue);
  }

  public void addValidation(SimpleTypeSG pController, JavaMethod pMethod, DirectAccessible pValue)
      throws SAXException {
    AtomicTypeSG atomicType = pController.getAtomicType();
    Long length = atomicType.getLength();
    Long maxLength = atomicType.getMaxLength();
    Long minLength = atomicType.getMinLength();

    if (minLength != null  &&  minLength.longValue() < 0) {
      throw new LocSAXException("Negative value for minLength detected: " + minLength, getLocator());
    }
    if (maxLength != null) { 
      if (maxLength.longValue() < 0) {
        throw new LocSAXException("Negative value for maxLength detected: " + maxLength, getLocator());
      }
      if (minLength != null) {
        if (maxLength.longValue() < minLength.longValue()) {
          throw new LocSAXException("MaxLength value of " + maxLength + " is smaller than minLength value of " + minLength,
                                     getLocator());
        }
        if (maxLength.longValue() == minLength.longValue()) {
          length = maxLength;
        }
      }
    }
    if (length != null) { 
      if (length.longValue() < 0) {
        throw new LocSAXException("Negative value for length detected: " + length, getLocator());
      }
      if (maxLength != null) {
        if (maxLength.longValue() < length.longValue()) {
          throw new LocSAXException("MaxLength value of " + maxLength + " is smaller than length value of " + length,
                                     getLocator());
        }
        maxLength = null;  // Avoid unnecessary checks
      }
      if (minLength != null) { 
        if (minLength.longValue() > length.longValue()) { 
          throw new LocSAXException("MinLength value of " + minLength + " is larger than length value of " + length,
                                     getLocator());
        }
        minLength = null;  // Avoid unnecessary checks
      }
    }

    if (length != null  ||  maxLength != null  ||  minLength != null) {
      if (pValue.isNullable()) {
        pMethod.addIf(pValue, " != null");
      }
      if (maxLength != null) {
        pMethod.addIf(pValue, ".length()", " > ", maxLength);
        pMethod.addThrowNew(IllegalArgumentException.class,
                            JavaSource.getQuoted("Length of " + maxLength + " characters exceeded: "),
                            " + ", pValue);
        pMethod.addEndIf();
      }
      if (minLength != null) {
        pMethod.addIf(pValue, ".length()", " < ", minLength);
        pMethod.addThrowNew(IllegalArgumentException.class,
                            JavaSource.getQuoted("Length of " + minLength + " characters exceeded: "),
                            " + ", pValue);
        pMethod.addEndIf();
      }
      if (length != null) {
        pMethod.addIf(pValue, ".length()", " != ", length);
        pMethod.addThrowNew(IllegalArgumentException.class,
                            JavaSource.getQuoted("Length of " + length + " characters not matched: "),
                            " + ", pValue);
        pMethod.addEndIf();
      }
      if (pValue.isNullable()) {
        pMethod.addEndIf();
      }
    }
  }
}
