/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.generator.util;



/** <p>A set of utility methods for using Java reflection.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann
 */
public class Reflect {
  /** <p>Assigns the value <code>pValue</code> to the beans
   * <code>pBean</code> property <code>pPropertyName</code>.
   * For example, if the property name is <samp>type</samp>
   * and the String pValue is <samp>2</samp>, calls
   * <pre>
   *   pBean.setType("2");
   * </pre>
   * or
   * <pre>
   *   pBean.setType(2);
   * </pre>
   * if the respective method exists. Does nothing, if no
   * matching method can be found.</p>
   */
  public static void assignBeanValue(Object pBean,
                                       String pPropertyName,
                                       String pValue)
      throws IllegalAccessException, java.lang.reflect.InvocationTargetException {
    assignBeanValue(pBean, pPropertyName, pValue, null);
  }

  /** <p>Assigns the value <code>pValue</code> to the beans
   * <code>pBean</code> property <code>pPropertyName</code>.
   * For example, if the property name is <samp>type</samp>
   * and the String pValue is <samp>2</samp>, calls
   * <pre>
   *   pBean.setType("2");
   * </pre>
   * or
   * <pre>
   *   pBean.setType(2);
   * </pre>
   * if the respective method exists. Does nothing, if no
   * matching method can be found.</p>
   */
  public static void assignBeanValue(Object pBean,
                                       String pPropertyName,
                                       String pValue,
                                       ReflectResolver pResolver)
      throws IllegalAccessException, java.lang.reflect.InvocationTargetException {
    Class c = pBean.getClass();
    java.lang.reflect.Method[] methods = c.getMethods();
    String methodName = "set" + Character.toUpperCase(pPropertyName.charAt(0)) +
      pPropertyName.substring(1);
    for (int i = 0;  i < methods.length;  i++) {
      java.lang.reflect.Method m = methods[i];
      if (methodName.equals(m.getName())) {
        Class[] parameterTypes = m.getParameterTypes();
        if (parameterTypes.length == 1) {
          Class parameterClass = parameterTypes[0];
          Object[] parameters = null;
          if (pResolver != null) {
            parameters = pResolver.resolve(parameterClass, pValue);
          }
          if (parameters == null) {
            if (parameterClass == String.class) {
              parameters = new Object[]{pValue};
            } else if (parameterClass == Long.class  ||
                        parameterClass == long.class) {
              parameters = new Object[]{new Long(pValue)};
            } else if (parameterClass == Integer.class  ||
                        parameterClass == int.class) {
              parameters = new Object[]{new Integer(pValue)};
            } else if (parameterClass == Short.class  ||
                        parameterClass == short.class) {
              parameters = new Object[]{new Short(pValue)};
            } else if (parameterClass == Byte.class  ||
                        parameterClass == byte.class) {
              parameters = new Object[]{new Byte(pValue)};
            } else if (parameterClass == Double.class  ||
                        parameterClass == double.class) {
              parameters = new Object[]{new Double(pValue)};
            } else if (parameterClass == Float.class  ||
                        parameterClass == float.class) {
              parameters = new Object[]{new Float(pValue)};
            } else if (parameterClass == Boolean.class  ||
                        parameterClass == boolean.class) {
              parameters = new Object[]{new Boolean(pValue)};
            }
          }
          if (parameters != null) {
            m.invoke(pBean, parameters);
            return;
          }
        }
      }
    }
  }
}
