/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.impl;

import java.text.Format;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.DatatypeConverterInterface;
import javax.xml.bind.PropertyException;
import javax.xml.bind.ValidationEventHandler;

import org.apache.ws.jaxme.xs.util.XsDateFormat;
import org.apache.ws.jaxme.xs.util.XsDateTimeFormat;
import org.apache.ws.jaxme.xs.util.XsTimeFormat;



/** <p>Common subclass for JMMarshallerImpl, JMUnmarshallerImpl and
 * JMValidatorImpl.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: JMControllerImpl.java 231712 2004-01-24 22:14:39Z jochen $
 */
public class JMControllerImpl {
  /** <p>Property prefix for users private settings:
   * "jaxme.private."; if a property
   * name starts with this prefix, then the property value is
   * stored in an internal Map.</p>
   */
  public static final String JAXME_PRIVATE = "jaxme.private.";

  /** <p>Name of the property for setting the
   * {@link javax.xml.bind.DatatypeConverterInterface}: "jaxme.datatypeConverter"</p>
   */
  public static final String JAXME_DATATYPE_CONVERTER = "jaxme.datatypeConverter";

  /** <p>Property for setting an instance of {@link java.text.Format},
   * which is being used for parsing and printing <code>xs:dateTime</code>
   * values. Defaults to an instance of
   * {@link org.apache.ws.jaxme.xs.util.XsDateTimeFormat}.</p>
   */
  public static final String JAXME_FORMAT_DATETIME = "jaxme.format.dateTime";

  /** <p>Property for setting an instance of {@link java.text.Format},
   * which is being used for parsing and printing <code>xs:date</code>
   * values. Defaults to an instance of
   * {@link org.apache.ws.jaxme.xs.util.XsDateFormat}.</p>
   */
  public static final String JAXME_FORMAT_DATE = "jaxme.format.date";

  /** <p>Property for setting an instance of {@link java.text.Format},
   * which is being used for parsing and printing <code>xs:time</code>
   * values. Defaults to an instance of
   * {@link org.apache.ws.jaxme.xs.util.XsTimeFormat}.</p>
   */
  public static final String JAXME_FORMAT_TIME = "jaxme.format.time";

  private Map privateMap;
  private JAXBContextImpl context;
  private DatatypeConverterInterface datatypeConverter = new DatatypeConverterImpl();

  public void setProperty(String pProperty, Object pValue)
      throws PropertyException {
    if (pProperty.startsWith(JAXME_PRIVATE)) {
      if (privateMap == null) {
        privateMap = new HashMap();
      }
      privateMap.put(pProperty, pValue);
      return;
    } else if (pProperty.equals(JAXME_DATATYPE_CONVERTER)) {
      datatypeConverter = (DatatypeConverterInterface) pValue;
      return;
    } else if (JAXME_FORMAT_DATETIME.equals(pProperty)) {
      setDateTimeFormat((Format) pValue);
    } else if (JAXME_FORMAT_DATE.equals(pProperty)) {
      setDateFormat((Format) pValue);
    } else if (JAXME_FORMAT_TIME.equals(pProperty)) {
      setTimeFormat((Format) pValue);
    }
    
    throw new PropertyException("Unknown property: " + pProperty);
  }

  public Object getProperty(String pProperty) throws PropertyException {
    if (pProperty.startsWith(JAXME_PRIVATE)) {
      if (privateMap == null) { return null; }
      return privateMap.get(pProperty);
    } else if (pProperty.equals(JAXME_DATATYPE_CONVERTER)) {
      return datatypeConverter;
    } else if (JAXME_FORMAT_DATETIME.equals(pProperty)) {
      return getDateTimeFormat();
    } else if (JAXME_FORMAT_DATE.equals(pProperty)) {
      return getDateFormat();
    } else if (JAXME_FORMAT_TIME.equals(pProperty)) {
      return getTimeFormat();
    }

    throw new PropertyException("Unknown property: " + pProperty);
  }

  protected ValidationEventHandler eventHandler;

  public ValidationEventHandler getEventHandler() { return eventHandler; }

  public void setEventHandler(ValidationEventHandler pEventHandler) { eventHandler = pEventHandler; }

  public void setJAXBContextImpl(JAXBContextImpl pContext) { context = pContext; }
  public JAXBContextImpl getJAXBContextImpl() { return context; }

  public void setDatatypeConverter(DatatypeConverterInterface pConverter) { datatypeConverter = pConverter; }
  public DatatypeConverterInterface getDatatypeConverter() { return datatypeConverter; }

private Format dateTimeFormat;

private Format dateFormat;

private Format timeFormat;

/** <p>Sets the {@link java.text.Format} for parsing and printing
   * <code>xs:dateTime</code> values.</p>
   * @param pFormat An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsDateTimeFormat}
   *   (default).
   */
public void setDateTimeFormat(Format pFormat) {
      dateTimeFormat = pFormat;
  }

/** <p>Returns the {@link java.text.Format} for parsing and printing
   * <code>xs:dateTime</code> values.</p>
   * @return An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsDateTimeFormat}
   *   (default).
   */
public Format getDateTimeFormat() {
      if (dateTimeFormat == null) {
          dateTimeFormat = new XsDateTimeFormat();
      }
      return dateTimeFormat;
  }

/** <p>Sets the {@link java.text.Format} for parsing and printing
   * <code>xs:date</code> values.</p>
   * @param pFormat An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsDateFormat}
   *   (default).
   */
public void setDateFormat(Format pFormat) {
      dateFormat = pFormat;
  }

/** <p>Returns the {@link java.text.Format} for parsing and printing
   * <code>xs:date</code> values.</p>
   * @return An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsDateFormat}
   *   (default).
   */
public Format getDateFormat() {
      if (dateFormat == null) {
          dateFormat = new XsDateFormat();
      }
      return dateFormat;
  }

/** <p>Sets the {@link java.text.Format} for parsing and printing
   * <code>xs:date</code> values.</p>
   * @param pFormat An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsDateFormat}
   *   (default).
   */
public void setTimeFormat(Format pFormat) {
      timeFormat = pFormat;
  }

/** <p>Returns the {@link java.text.Format} for parsing and printing
   * <code>xs:time</code> values.</p>
   * @return An instance of {@link java.text.DateFormat} or an
   *   instance of {@link org.apache.ws.jaxme.xs.util.XsTimeFormat}
   *   (default).
   */
public Format getTimeFormat() {
      if (timeFormat == null) {
          timeFormat = new XsTimeFormat();
      }
      return timeFormat;
  }
}
