/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003,2004 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.impl;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBException;
import javax.xml.bind.UnmarshallerHandler;
import javax.xml.namespace.QName;

import org.apache.ws.jaxme.JMHandler;
import org.apache.ws.jaxme.JMUnmarshaller;
import org.apache.ws.jaxme.Observer;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/** <p>Implementation of a JMUnmarshallerHandler; the
 * UnmarshallerHandler receives SAX events which he silently
 * discards, as long as the first <code>startElement</code>
 * event is seen. Depending on namespace URI and local name,
 * the Unmarshallerhandler creates a new instance of JMHandler
 * and from now on forwards all SAX events to the JMHandler.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: JMUnmarshallerHandlerImpl.java 231705 2004-01-07 02:07:20Z jochen $
 */
public class JMUnmarshallerHandlerImpl implements UnmarshallerHandler,
    Observer {
  private JMUnmarshaller unmarshaller;
  private JMHandler myHandler;
  private Object result;
  private Locator locator;
  private List prefixes = new ArrayList();

  public void setJMUnmarshaller(JMUnmarshaller pUnmarshaller) {
    unmarshaller = pUnmarshaller;
  }

  public JMUnmarshaller getJMUnmarshaller() {
    return unmarshaller;
  }

  public void notify(Object pObject) {
    result = pObject;
  }

  public Object getResult() throws JAXBException, IllegalStateException {
    if (myHandler == null) {
      throw new IllegalStateException("No startElement event has been seen.");
    }
    if (result == null) {
      throw new IllegalStateException("Parsing is not yet finished.");
    }
    return result;
  }

  public void setDocumentLocator(Locator pLocator) {
    locator = pLocator;
    if (myHandler != null) { myHandler.setDocumentLocator(pLocator); }
  }

  public void startDocument() throws SAXException {
    result = null;
    myHandler = null;
  }

  public void endDocument() throws SAXException {
    if (myHandler != null) {
      myHandler.endDocument();
    }
  }

  public void startPrefixMapping(String pPrefix, String pURI)
      throws SAXException {
    if (myHandler != null) {
      myHandler.startPrefixMapping(pPrefix, pURI);
    } else {
      prefixes.add(pPrefix);
      prefixes.add(pURI);
    }
  }

  public void endPrefixMapping(String pPrefix) throws SAXException {
    if (myHandler != null) {
      myHandler.endPrefixMapping(pPrefix);
    } else {
      for (int i = prefixes.size()-2;  i >= 0;  i -= 2) {
          if (pPrefix.equals(prefixes.get(i))) {
              prefixes.remove(i);
              prefixes.remove(i);
              return;
          }
      }
      throw new SAXException("Undeclared namespace prefix: " + pPrefix);
    }
  }

  public void startElement(String pNamespaceURI, String pLocalName,
                            String pQName, Attributes pAttr) throws SAXException {
    if (myHandler == null) {
      QName name = new QName(pNamespaceURI, pLocalName);
      try {
        myHandler = unmarshaller.getJAXBContextImpl().getJMHandler(name);
        myHandler.init(unmarshaller);
      } catch (JAXBException e) {
        throw new SAXException(e);
      }
      if (locator != null) {
        myHandler.setDocumentLocator(locator);
      }
      myHandler.setObserver(this);
      myHandler.startDocument();
      if (prefixes.size() > 0) {
        for (int i = 0;  i < prefixes.size();  i+=2) {
          myHandler.startPrefixMapping(prefixes.get(i).toString(),
                                       prefixes.get(i+1).toString());
        }
        prefixes.clear();
      }
    }
    myHandler.startElement(pNamespaceURI, pLocalName, pQName, pAttr);
  }

  public void endElement(String pNamespaceURI, String pLocalName, String pQName)
      throws SAXException {
    myHandler.endElement(pNamespaceURI, pLocalName, pQName);
  }

  public void characters(char[] pChars, int pStart, int pLen)
      throws SAXException {
    myHandler.characters(pChars, pStart, pLen);
  }

  public void ignorableWhitespace(char[] pChars, int pStart, int pLen)
      throws SAXException {
    myHandler.ignorableWhitespace(pChars, pStart, pLen);
  }

  public void processingInstruction(String pTarget, String pData)
      throws SAXException {
    if (myHandler != null) {
      myHandler.processingInstruction(pTarget, pData);
    }
  }

  public void skippedEntity(String pName) throws SAXException {
    if (myHandler != null) {
      myHandler.skippedEntity(pName);
    }
  }
}
