/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.net.URL;

import javax.xml.bind.JAXBException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.UnmarshallerHandler;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;

import org.apache.ws.jaxme.JMUnmarshaller;
import org.apache.ws.jaxme.util.DOMSerializer;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: JMUnmarshallerImpl.java 231559 2003-09-23 12:37:47Z jochen $
 */
public class JMUnmarshallerImpl extends JMControllerImpl implements JMUnmarshaller {
  private static final SAXParserFactory spf;
  static {
    spf = SAXParserFactory.newInstance();
    spf.setValidating(false);
    spf.setNamespaceAware(true);
  }

  private boolean validating;

  public boolean isValidating() { return validating; }
  public void setValidating(boolean pValidating) { validating = pValidating; }

  public Object unmarshal(URL pURL) throws JAXBException {
    InputSource isource;
    try {
      isource = new InputSource(pURL.openStream());
      isource.setSystemId(pURL.toString());
    } catch (IOException e) {
      throw new UnmarshalException("Failed to open URL " + pURL, e);
    }
    return unmarshal(isource);
  }

  public Object unmarshal(File pFile) throws JAXBException {
    InputSource isource;
    try {
      isource = new InputSource(new FileInputStream(pFile));
      isource.setSystemId(pFile.toURL().toString());
    } catch (IOException e) {
      throw new UnmarshalException("Failed to open file " + pFile, e);
    }
    return unmarshal(isource);
  }

  public Object unmarshal(InputStream pStream) throws JAXBException {
    return unmarshal(new InputSource(pStream));
  }

  public Object unmarshal(Reader pReader) throws JAXBException {
    return unmarshal(new InputSource(pReader));
  }

  public Object unmarshal(InputSource pSource) throws JAXBException {
    UnmarshallerHandler uh = getUnmarshallerHandler();
    try {
      SAXParser sp = spf.newSAXParser();
      XMLReader xr = sp.getXMLReader();
      xr.setContentHandler(uh);
      xr.parse(pSource);
    } catch (SAXException e) {
      if (e.getException() != null) {
        throw new UnmarshalException(e.getException());
      } else {
        throw new UnmarshalException(e);
      }
    } catch (IOException e) {
      throw new UnmarshalException(e);
    } catch (ParserConfigurationException e) {
      throw new UnmarshalException(e);
    }
    return uh.getResult();
  }

  public Object unmarshal(Node pNode) throws JAXBException {
    UnmarshallerHandler uh = getUnmarshallerHandler();
    DOMSerializer ds = new DOMSerializer();
    try {
      ds.serialize(pNode, uh);
    } catch (SAXException e) {
      if (e.getException() != null) {
        throw new UnmarshalException(e.getException());
      } else {
        throw new UnmarshalException(e);
      }
    }
    return uh.getResult();
  }

  public Object unmarshal(Source pSource) throws JAXBException {
    if (pSource instanceof SAXSource) {
      SAXSource ss = (SAXSource) pSource;
      InputSource is = ss.getInputSource();
      if (is == null) {
        throw new UnmarshalException("The SAXResult doesn't have its InputSource set.");
      }
      XMLReader xr = ss.getXMLReader();
      if (xr == null) {
        return unmarshal(is);
      } else {
        UnmarshallerHandler uh = getUnmarshallerHandler();
        xr.setContentHandler(uh);
        try {
          xr.parse(is);
        } catch (IOException e) {
          throw new JAXBException(e);
        } catch (SAXException e) {
          if (e.getException() != null) {
            throw new JAXBException(e.getException());
          } else {
            throw new JAXBException(e);
          }
        }
        return uh.getResult();
      }
    } else if (pSource instanceof StreamSource) {
      StreamSource ss = (StreamSource) pSource;
      InputSource iSource = new InputSource();
      iSource.setPublicId(ss.getPublicId());
      iSource.setSystemId(ss.getSystemId());
      Reader r = ss.getReader();
      if (r == null) {
        InputStream is = ss.getInputStream();
        if (is == null) {
          throw new IllegalArgumentException("The StreamSource doesn't have its Reader or InputStream set.");
        } else {
          iSource.setByteStream(is);
        }
      } else {
        iSource.setCharacterStream(r);
      }
      return unmarshal(iSource);
    } else if (pSource instanceof DOMSource) {
      Node node = ((DOMSource) pSource).getNode();
      if (node == null) {
        throw new IllegalArgumentException("The DOMSource doesn't have its Node set.");
      }
      return unmarshal(node);
    } else {
      throw new IllegalArgumentException("Unknown type of Source: " + pSource.getClass().getName() +
                                          ", only SAXSource, StreamSource and DOMSource are supported.");
    }
  }

  public UnmarshallerHandler getUnmarshallerHandler() {
    JMUnmarshallerHandlerImpl result = new JMUnmarshallerHandlerImpl();
    result.setJMUnmarshaller(this);
    return result;
  }
}
