/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.util;


/** <p>Helper class for working with class loaders.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: ClassLoader.java 231597 2003-10-20 19:48:06Z jochen $
 */
public class ClassLoader {
  /** <p>Loads a class with the given name. First attempts to use
   * the context class loader, then its own class loader.</p>
   *
   * @param pName The fully qualified name of the class being loaded.
   * @throws ClassNotFoundException Loading the class failed.
   * @return The class with the name <code>pName</code>.
   */
  public static Class getClass(String pName) throws ClassNotFoundException {
    // First try: The default ClassLoader
    try {
      return Class.forName(pName);
    } catch (ClassNotFoundException e) {
      // Second try: The threads context ClassLoader
      try {
        return Thread.currentThread().getContextClassLoader().loadClass(pName);
      } catch (ClassNotFoundException f) {
        throw e;
      }
    }
  }

  /** <p>Loads a class with the given name using <code>getClass(String)</code>.
   * If an instance of the returned class cannot be assigned to the
   * class or interface <code>pAssignableTo</code>, throws an
   * IllegalArgumentException.</p>
   *
   * @throws ClassNotFoundException The class with the name <code>pName</code>
   *   could not be loaded.
   * @throws IllegalArgumentException Instances of the class with the name
   *   <code>pName</code> are not assignable to the interface or class
   *   <code>pAssignableTo</code>.
   */
  public static Class getClass(String pName, Class pAssignableTo)
      throws ClassNotFoundException {
    Class result = getClass(pName);
    if (pAssignableTo != null  &&  !pAssignableTo.isAssignableFrom(result)) {
      throw new IllegalArgumentException("The class " + result.getName() +
                                          " is not implementing or extending " +
                                          pAssignableTo.getName());
    }
    return result;
  }
}
