/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003,2004 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;


/** <p>Similar to org.xml.sax.NamespaceSupport, but
 * for marshalling and not for parsing XML.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: NamespaceSupport.java 231705 2004-01-07 02:07:20Z jochen $
 */
public class NamespaceSupport implements NamespaceContext {
    List prefixList;
    String cachedURI, cachedPrefix;
    
    /** <p>Creates a new instance of NamespaceSupport.</p> */
    public NamespaceSupport() {
    }
    
    /** <p>Resets the NamespaceSupport's state for reuse.</p>
     */
    public void reset() {
        cachedURI = cachedPrefix = null;
        if (prefixList != null) {
            prefixList.clear();
        }
    }
    
    /** <p>Declares a new prefix.</p>
     */
    public void declarePrefix(String pPrefix, String pURI) {
        if (pURI == null) { pURI = ""; }
        if (cachedURI == null) {
            cachedURI = pURI;
            cachedPrefix = pPrefix;
        } else {
            if (prefixList == null) { prefixList = new ArrayList(); }
            prefixList.add(cachedPrefix);
            prefixList.add(cachedURI);
            cachedPrefix = pPrefix;
            cachedURI = pURI;
        }
    }
    
    /** <p>Removes a prefix declaration. Assumes that the prefix is the
     * current prefix. If not, throws a IllegalStateException.</p>
     */
    public void undeclarePrefix(String pPrefix) {
        if (pPrefix.equals(cachedPrefix)) {
            if (prefixList != null  &&  prefixList.size() > 0) {
                cachedURI = prefixList.remove(prefixList.size()-1).toString();
                cachedPrefix = prefixList.remove(prefixList.size()-1).toString();
            } else {
                cachedPrefix = cachedURI = null;
            }
        } else {
            for (int i = prefixList.size()-2;  i >= 0;  i -= 2) {
                if (pPrefix.equals(prefixList.get(i))) {
                    prefixList.remove(i);
                    prefixList.remove(i);
                    return;
                }
            }
            throw new IllegalStateException("Undeclared prefix: " + pPrefix);
        }
    }
    
    /** <p>Given a prefix, returns the URI to which the prefix is
     * currently mapped or null, if there is no such mapping.</p>
     * <p><em>Note</em>: This methods behaviour is precisely
     * defined by {@link NamespaceContext#getNamespaceURI(java.lang.String)}.</p>
     *
     * @param pPrefix The prefix in question
     */
    public String getNamespaceURI(String pPrefix) {
        if (pPrefix == null) {
            throw new IllegalArgumentException("Namespace prefix must not be null");
        }
        if (cachedURI != null) {
            if (cachedPrefix.equals(pPrefix)) { return cachedURI; }
            if (prefixList != null) {
                for (int i = prefixList.size();  i > 0;  i -= 2) {
                    if (pPrefix.equals(prefixList.get(i-2))) {
                        return (String) prefixList.get(i-1);
                    }
                }
            }
        }
        if (XMLConstants.XML_NS_PREFIX.equals(pPrefix)) {
            return XMLConstants.XML_NS_URI;
        } else if (XMLConstants.XMLNS_ATTRIBUTE.equals(pPrefix)) {
            return XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
        }
        return null;
    }
    
    /** <p>Returns a prefix currently mapped to the given URI or
     * null, if there is no such mapping. This method may be used
     * to find a possible prefix for an elements namespace URI. For
     * attributes you should use {@link #getAttributePrefix(String)}.</p>
     * <p><em>Note</em>: This methods behaviour is precisely
     * defined by {@link NamespaceContext#getPrefix(java.lang.String)}.</p>
     *
     * @param pURI The namespace URI in question
     * @throws IllegalArgumentException The namespace URI is null.
     */
    public String getPrefix(String pURI) {
        if (pURI == null) {
            throw new IllegalArgumentException("Namespace URI must not be null");
        }
        if (cachedURI != null) {
            if (cachedURI.equals(pURI)) { return cachedPrefix; }
            if (prefixList != null) {
                for (int i = prefixList.size();  i > 0;  i -= 2) {
                    if (pURI.equals(prefixList.get(i-1))) {
                        return (String) prefixList.get(i-2);
                    }
                }
            }
        }
        if (XMLConstants.XML_NS_URI.equals(pURI)) {
            return XMLConstants.XML_NS_PREFIX;
        } else if (XMLConstants.XMLNS_ATTRIBUTE_NS_URI.equals(pURI)) {
            return XMLConstants.XMLNS_ATTRIBUTE;
        }
        return null;
    }
    
    /** <p>Returns a non-empty prefix currently mapped to the given
     * URL or null, if there is no such mapping. This method may be
     * used to find a possible prefix for an attributes namespace
     * URI. For elements you should use {@link #getPrefix(String)}.</p>
     *
     * @param pURI Thhe namespace URI in question
     * @throws IllegalArgumentException The namespace URI is null.
     */
    public String getAttributePrefix(String pURI) {
        if (pURI == null) {
            throw new IllegalArgumentException("Namespace URI must not be null");
        }
        if (pURI == null  ||  pURI.length() == 0) {
            return "";
        }
        if (cachedURI != null) {
            if (cachedURI.equals(pURI)  &&  cachedPrefix.length() > 0) {
                return cachedPrefix;
            }
            if (prefixList != null) {
                for (int i = prefixList.size();  i > 0;  i -= 2) {
                    if (pURI.equals(prefixList.get(i-1))) {
                        String prefix = (String) prefixList.get(i-2);
                        if (prefix.length() > 0) {
                            return prefix;
                        }
                    }
                }
            }
        }    
        if (XMLConstants.XML_NS_URI.equals(pURI)) {
            return XMLConstants.XML_NS_PREFIX;
        } else if (XMLConstants.XMLNS_ATTRIBUTE_NS_URI.equals(pURI)) {
            return XMLConstants.XMLNS_ATTRIBUTE;
        }
        return null;
    }
    
    
    
    /** <p>Returns a collection to all prefixes bound to the given
     * namespace URI.
     * <p><em>Note</em>: This methods behaviour is precisely
     * defined by {@link NamespaceContext#getPrefixes(java.lang.String)}.</p>
     *
     * @param pURI The namespace prefix in question
     */
    public Iterator getPrefixes(String pURI) {
        if (pURI == null) {
            throw new IllegalArgumentException("Namespace URI must not be null");
        }
        List list = new ArrayList();
        if (cachedURI != null) {
            if (cachedURI.equals(pURI)) { list.add(cachedPrefix); }
            if (prefixList != null) {
                for (int i = prefixList.size();  i > 0;  i -= 2) {
                    if (pURI.equals(prefixList.get(i-1))) {
                        list.add(prefixList.get(i-2));
                    }
                }
            }
        }
        if (pURI.equals(XMLConstants.XMLNS_ATTRIBUTE_NS_URI)) {
            list.add(XMLConstants.XMLNS_ATTRIBUTE);
        } else if (pURI.equals(XMLConstants.XML_NS_URI)) {
            list.add(XMLConstants.XML_NS_PREFIX);
        }
        return list.iterator();
    }
    
    /** <p>Returns whether a given prefix is currently declared.</p>
     */
    public boolean isPrefixDeclared(String pPrefix) {
        if (cachedURI != null) {
            if (cachedPrefix != null  &&  cachedPrefix.equals(pPrefix)) { return true; }
            if (prefixList != null) {
                for (int i = prefixList.size();  i > 0;  i -= 2) {
                    if (prefixList.get(i-2).equals(pPrefix)) {
                        return true;
                    }
                }
            }
        }
        return "xml".equals(pPrefix);
    }
    
    public int getContext() {
        return (prefixList == null ? 0 : prefixList.size()) +
        (cachedURI == null ? 0 : 2);
    }
    
    public String setContext(int i) {
        int current = getContext();
        if (current == getContext()) {
            return null;
        }
        String result = cachedPrefix;
        if (prefixList != null  &&  prefixList.size() > 0) {
            cachedURI = prefixList.remove(prefixList.size()-1).toString();
            cachedPrefix = prefixList.remove(prefixList.size()-1).toString();
        } else {
            cachedURI = null;
            cachedPrefix = null;
        }
        return result;
    }
}
