/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.js;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.ws.jaxme.js.JavaSource.Protection;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public abstract class AbstractJavaMethod
    extends ConditionalIndentationJavaSourceObject {
  private List exceptions = new ArrayList();
  private List params = new ArrayList();
  public AbstractJavaMethod(String pName, JavaQName pType,
                            Protection pProtection) {
    super(pName, pType, pProtection);
  }

  /** <p>Returns whether the method is throwing the given exception.
   * Note that this method doesn't care for inheritance. For example,
   * if the method declares to be throwing an {@link java.io.IOException},
   * then the value <code>isThrowing(java.net.MalformedURLException.class)</code>
   * is still false.</p>
   */
  public boolean isThrowing(JavaQName e) {
    if (e == null) {
      throw new NullPointerException("The exception argument must not be null.");
    }
    for (Iterator iter = exceptions.iterator();  iter.hasNext();  ) {
      if (e.equals(iter.next())) {
        return true;
      }
    }
    return false;
  }

  /** <p>Returns whether the method is throwing the given exception.
   * Note that this method doesn't care for inheritance. For example,
   * if the method declares to be throwing an {@link java.io.IOException},
   * then the value <code>isThrowing(java.net.MalformedURLException.class)</code>
   * is still false.</p>
   */
  public boolean isThrowing(Class e) {
    if (e == null) {
      throw new NullPointerException("The exception argument must not be null.");
    }
    return isThrowing(JavaQNameImpl.getInstance(e));
  }

  /** <p>Adds an exception to this methods list of exceptions.</p>
   *
   * @see #getExceptions
   */
  public void addThrows(JavaQName e) {
    if (e == null) {
      throw new NullPointerException("The exception argument must not be null.");
    }
    exceptions.add(e);
  }

  /** <p>Adds an exception to this methods list of exceptions.</p>
   *
   * @see #getExceptions
   */
  public void addThrows(Class e) {
    if (e == null) {
      throw new NullPointerException("The exception argument must not be null.");
    }
    exceptions.add(JavaQNameImpl.getInstance(e));
  }

  /** <p>Adds an exception to this methods list of exceptions.</p>
   *
   * @see #getExceptions
   * @deprecated Use {@link #addThrows(JavaQName)}
   */
  public void addThrows(String e) {
    exceptions.add(JavaQNameImpl.getInstance(e));
    if (e == null) {
      throw new NullPointerException("The exception argument must not be null.");
    }
  }


  /** <p>Adds a parameter that this method takes.</p>
   *
   * @see #getParams
   * @deprecated Use {@link #addParam(JavaQName, String)}
   */
  public void addParam(String p) {
    if (p == null) {
      throw new NullPointerException("param argument must not be null");
    }
    StringTokenizer st = new StringTokenizer(p);
    if (!st.hasMoreTokens()) {
      throw new IllegalArgumentException("param argument must have two tokens: type name");
    }
    String type = st.nextToken();
    if (!st.hasMoreTokens()) {
      throw new IllegalArgumentException("param argument must have two tokens: type name");
    }
    String name = st.nextToken();
    if (st.hasMoreTokens()) {
      throw new IllegalArgumentException("param argument must have exactly two tokens: type name");
    }
    addParam(type, name);
  }

  /** <p>Adds a parameter that this method takes.</p>
   *
   * @see #getParams
   * @deprecated Use {@link #addParam(JavaQName, String)}
   */
  public void addParam(String p, String v) {
    if (p == null) {
      throw new NullPointerException("param argument must not be null");
    }
    p = p.trim();
    int offset = p.indexOf('.');
    if (offset == -1) {
      addParam(JavaQNameImpl.getInstance(p), v);
    } else {
      addParam(JavaQNameImpl.getInstance(p.substring(0, offset),
                                         p.substring(offset+1)), v);
    }
  }

  /** <p>Adds a parameter that this method takes.</p>
   *
   * @see #getParams
   * @return An object to use for referencing the parameter inside the method.
   */
  public Parameter addParam(Class p, String v) {
    return addParam(JavaQNameImpl.getInstance(p), v);
  }

  /** <p>Adds a parameter that this method takes.</p>
   *
   * @see #getParams
   * @return An object to use for referencing the parameter inside the method.
   */
  public Parameter addParam(JavaQName pType, String pName) {
    if (pType == null) {
      throw new NullPointerException("Type argument must not be null");
    }
    if (pName == null) {
      throw new NullPointerException("Parameter name argument must not be null");
    }
    Parameter p = new Parameter(pType, pName);
    params.add(p);
    return p;
  }

  /** <p>Adds a parameter that this method takes.</p>
   *
   * @see #getParams
   * @return An object to use for referencing the parameter inside the method.
   */
  public Parameter addParam(Parameter pParam) {
    return addParam(pParam.getType(), pParam.getName());
  }

  /** <p>Clears the list of parameters.</p>
   */
  public void clearParams() {
    params.clear();
  }

  /** <p>Returns the list of exceptions thrown by this method.</p>
   *
   * @see #addThrows(JavaQName)
   */
  public JavaQName[] getExceptions() {
    return (JavaQName[]) exceptions.toArray(new JavaQName[exceptions.size()]);
  }


  /** <p>Returns the list of parameters that this method takes. Any element
   * in the list is an instance of {@link Parameter}.</p>
   *
   * @see #addParam(JavaQName, String)
   */
  public Parameter[] getParams() {
         return (Parameter[]) params.toArray(new Parameter[params.size()]);
  }
}
