/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.js;

/** <p>This interface is an abstract base for fields and similar
 * objects. It doesn't define many methods, it mainly indicates, that
 * the implementations value is directly and fast accessible in
 * the generated code. The use is best demonstrated by an example.
 * Suggest the following piece of code:</p>
 * <pre>
 *     Object value;
 *     return new Object[]{"((", value, ") * (", value, "))"};
 * </pre>
 * <p>The example is well suited for the case, where <code>value</code>
 * is a variable name like "i". It is not suited, if "value" contains
 * an expensive method call like "sin(x)". It is even wrong in the
 * case "i++".</p>
 * <p>By using the interface <code>DirectAccessible</code>, you can
 * change the implementation of <code>getSquare()</code> to look
 * like this:
 * <pre>
 *     Object value;
 *     JavaQName type;
 *     if (!(value instanceof DirectAccessible)) {
 *       LocalJavaField v = pMethod.newJavaField(type);
 *       v.addLine(value);
 *       v.setFinal(true);
 *       value = v;
 *     }
 *     return new Object[]{"((", value, ") * (", value, "))"};
 * </pre>
 * <p>This results in code, which is far more readable and better
 * optimized.</p>
 *
 * @author <a href="mailto:jwi@softwareag.com">Jochen Wiedmann</a>
 */
public interface DirectAccessible {
   /** <p>Returns whether the value is possibly null.</p>
    */
   public boolean isNullable();
   /** <p>Sets whether the value is possibly null.</p>
    */
   public void setNullable(boolean pNullable);
}
