/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.js;

import java.io.IOException;


/** <p>Implements a Java method.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class JavaMethod extends AbstractJavaMethod {
  private boolean isSynchronized;

  /** <p>Creates a new JavaMethod with the given name, return
   * type and protection.</p>
   */
  JavaMethod(String pName, JavaQName pType, JavaSource.Protection pProtection) {
    super(pName, pType, pProtection);
  }

  /** <p>Sets whether this method is synchronized.</p>
   */
  public void setSynchronized(boolean pSynchronized) {
    isSynchronized = pSynchronized;
  }

  /** <p>Returns whether this method is synchronized.</p>
	*/
  public boolean isSynchronized() {
    return isSynchronized;
  }

  /** <p>Adds a header line.</p>
   */
  protected void writeHeader(IndentationTarget pTarget) throws IOException {
    pTarget.indent(0);
    JavaSource.Protection protection = getProtection();
    if (protection != null  &&  !protection.equals(JavaSource.DEFAULT_PROTECTION)) {
      pTarget.write(getProtection().toString());
      pTarget.write(" ");
    }
    if (isStatic()) {
      pTarget.write("static ");
    }
    if (isFinal()) {
      pTarget.write("final ");
    }
    if (isAbstract()) {
      pTarget.write("abstract ");
    }
    if (isSynchronized()  &&  !pTarget.isInterface()) {
    	pTarget.write("synchronized ");
    }
	 pTarget.write(pTarget.asString(getType()));
    pTarget.write(" ");
    pTarget.write(getName());
    pTarget.write("(");
    Parameter[] params = getParams();
    for (int i = 0;  i < params.length;  i++) {
      if (i > 0) {
        pTarget.write(", ");
      }
      Parameter p = params[i];
      pTarget.write(pTarget.asString(p.getType()) );
      pTarget.write(" ");
      pTarget.write(p.getName());
    }
    pTarget.write(")");
    JavaQName[] exceptions = getExceptions();
    for (int i = 0;  i < exceptions.length; i++) {
        if (i > 0) {
          pTarget.write(", ");
        } else {
          pTarget.write(" throws ");
        }
      pTarget.write(pTarget.asString(exceptions[i]));
    }
    if (pTarget.isInterface()  ||  isAbstract()) {
      pTarget.write(";");
    } else {
      pTarget.write(" {");
    }
    pTarget.write();
  }

  /** <p>Returns the abbreviated method signature: The method name, followed
   * by the parameter types. This is typically used in logging statements.</p>
   */
  public String getLoggingSignature() {
    StringBuffer result = new StringBuffer(getName());
    Parameter[] params = getParams();
    if (params.length > 0) {
      result.append('(');
      for (int i = 0; i < params.length; i++) {
        if (i > 0) { result.append(','); }
        result.append(params[i].getType().getClassName());
      }
      result.append(')');
    }
    return result.toString();
  }

  /** <p>Returns whether the JavaMethod is void.</p>
   */
  public boolean isVoid() {
    return getType().equals(JavaQNameImpl.VOID);
  }
}
