/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.js;

/** <p>Various static utility methods.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class Util {
	/** <p>Returns whether the given name is a valid Java identifier.
	 * Works by using {@link Character#isJavaIdentifierStart(char)} and
	 * {@link Character#isJavaIdentifierPart(char)}.</p>
	 * @throws IllegalArgumentException The name is not valid. An explanation
	 *   why is given in the detail message.
	 */
	public static void checkJavaIdentifier(String pName) {
		if (pName.length() == 0) {
			throw new IllegalArgumentException("A valid Java identifier must not be empty.");
		}
		char c = pName.charAt(0);
		if (!Character.isJavaIdentifierStart(c)) {
			throw new IllegalArgumentException("The identifier " + pName +
			                                    " is no valid Java identifier, because its first character is " + c);
		}
		for (int i = 1;  i < pName.length();  i++) {
			if (!Character.isJavaIdentifierPart(c)) {
				throw new IllegalArgumentException("The identifier " + pName +
				                                    " is no valid Java identifier, because it contains the character " + c);
			}
		}
	}

	/** <p>Takes as input an arbitrary String and maps it to a String,
	 * which is a valid Java identifier. Mapping works as follows:
	 * <ol>
	 *   <li>For the first character, invokes {@link Character#isJavaIdentifierStart(char)}.
	 *     If that method returns false, replaces the character with an
	 *     underscore ('_').</li>
	 *   <li>For any following character, invokes {@link Character#isJavaIdentifierPart(char)}.
	 *     If that method returns false, replaces the character with an
	 *     underscore ('_').</li>
	 * </ol>
	 * </p>
	 * @param pIdentifier The identifier being mapped
	 * @throws IllegalArgumentException The parameter <code>pIdentifier</code>
	 *   cannot be converted into a Java identifier, because it is null or
	 *   empty. 
	 */
	public static String asJavaIdentifier(String pIdentifier) {
		if (pIdentifier == null  ||  pIdentifier.length() == 0) {
			throw new IllegalArgumentException("A null or empty String cannot be converted into a valid Java identifier."); 
		}
		StringBuffer sb = new StringBuffer();
		char c = pIdentifier.charAt(0);
		sb.append(Character.isJavaIdentifierStart(c) ? c : '_');
		for (int i = 1;  i < pIdentifier.length();  i++) {
			c = pIdentifier.charAt(i);
			sb.append(Character.isJavaIdentifierPart(c) ? c : '_');
		}
		return sb.toString();
	}
}
