/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.logging;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;


/** <p>This class implements access to the Loggers through static
 * methods. The class typically configures itself from the
 * environment. However, you may choose to configure the class
 * explicitly by invoking {@link #setLoggerFactory(LoggerFactory)}.
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class LoggerAccess {
   private static LoggerFactory theFactory;

   /** <p>Sets the logger factory.</p>
    */
   public static synchronized void setLoggerFactory(LoggerFactory pFactory) {
      theFactory = pFactory;
   }

   /** <p>Instantiates the given {@link LoggerFactory}.</p>
    */
   private static LoggerFactory newLoggerFactory(String pName) {
      Class c = null;
      Throwable t = null;
      try {
         c = Class.forName(pName);
      } catch (Throwable th) {
         t = th;
      }
      if (c == null) {
         try {
            c = Thread.currentThread().getContextClassLoader().loadClass(pName);
         } catch (Throwable th) {
            if (t == null) {
               t = th;
            }
         }
      }
      if (c == null) {
         if (t == null) {
            t = new ClassNotFoundException(pName);
         }
         t.printStackTrace();
         return null;
      }

      try {
         return (LoggerFactory) c.newInstance();
      } catch (Throwable th) {
         t.printStackTrace();
         return null;
      }
   }

   /** <p>Creates a new instance of {@link LoggerFactory}. The
    * implementation class is determined as follows:
    * <ol>
    *   <li>If the system property <code>org.apache.ws.jaxme.logging.LoggerFactory</code>
    *     is set, uses the given class name.</li>
    *   <li>If the resource
    *     <code>META-INF/services/org.apache.ws.jaxme.logging.LoggerFactory</code>
    *     exists, uses the given class name.</li>
    *   <li>Otherwise returns a default instance logging to
    *     <code>System.err</code>.</li>
    * </ol>
    */
   public static LoggerFactory newLoggerFactory() {
      String p = LoggerFactory.class.getName();
      String v = System.getProperty(p);
      LoggerFactory result;
      if (v != null) {
         result = newLoggerFactory(v);
         if (result != null) {
            return result; 
         }
      }

      String res = "META-INF/services/" + p;
      URL url = LoggerAccess.class.getClassLoader().getResource(res);
      if (url == null) {
         url = Thread.currentThread().getContextClassLoader().getResource(res);
      }
      if (url != null) {
         try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
            v = reader.readLine();
            if (v != null) {
               result = newLoggerFactory(v);
            }
         } catch (Throwable t) {
            t.printStackTrace();
         }
      }

      return new LoggerFactoryImpl() {
         public Logger newLogger(String pName) {
            return new LoggerImpl(pName);
         }
      };
   }

   /** <p>Returns the logger factory. If a logger factory is set (by
    * previous calls to {@link #newLoggerFactory()} or
    * {@link #setLoggerFactory(LoggerFactory)}), returns that factory. Otherwise
    * invokes these methods and returns the result.</p>
    */
   public static synchronized LoggerFactory getLoggerFactory() {
      if (theFactory == null) {
         setLoggerFactory(newLoggerFactory());
      }
      return theFactory;
   }

   /** <p>Returns a new logger with the given name.</p>
    */
   public static synchronized Logger getLogger(String pName) {
      LoggerFactory factory = getLoggerFactory();
      return factory.getLogger(pName);
   }

   /** <p>Shortcut for <code>getLogger(pClass.getName())</code>.</p>
    */
   public static synchronized Logger getLogger(Class pClass) {
      return getLogger(pClass.getName());
   }
}
