/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls;

import java.util.Iterator;
import java.util.Map;

import org.apache.ws.jaxme.sqls.impl.CombinedConstraintImpl;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface CombinedConstraint extends Constraint {
   public interface Type {
      public final static Type AND = new CombinedConstraintImpl.TypeImpl("AND");
      public final static Type OR = new CombinedConstraintImpl.TypeImpl("OR");
   }

   /** <p>Returns the type, either of
    * {@link org.apache.ws.jaxme.sqls.CombinedConstraint.Type#AND} or
    * {@link org.apache.ws.jaxme.sqls.CombinedConstraint.Type#OR}.</p>
    */
   public CombinedConstraint.Type getType();

   /** <p>Creates an AndConstraint and inserts it at the current position.</p>
    */
   public CombinedConstraint createAndConstraint();

   /** <p>Creates an OrConstraint and inserts it at the current position.</p>
    */
   public CombinedConstraint createOrConstraint();

   /** <p>Creates a condition "equals" (=) and inserts it at the current position.</p>
    */
   public BooleanConstraint createEQ();

   /** <p>Creates a condition "doesn't equal" (!=) and inserts it at the current position.</p>
    */
   public BooleanConstraint createNE();

   /** <p>Creates a condition "lower than" (<) and inserts it at the current position.</p>
    */
   public BooleanConstraint createLT();

   /** <p>Creates a condition "greater than" (<) and inserts it at the current position.</p>
    */
   public BooleanConstraint createGT();

   /** <p>Creates a condition "lower or equals" (<=) and inserts it at the current position.</p>
    */
   public BooleanConstraint createLE();

   /** <p>Creates a condition "greater or equals" (>=) and inserts it at the current position.</p>
    */
   public BooleanConstraint createGE();

   /** <p>Creates a condition "LIKE" and inserts it at the current position.</p>
    */
   public BooleanConstraint createLIKE();

   /** <p>Creates a condition "IS NULL" and inserts it at the current position.</p>
    */
   public BooleanConstraint createISNULL();

   /** <p>Creates a condition "IN" and inserts it at the current position.</p>
    */
   public BooleanConstraint createIN();

   /** <p>Creates a condition "EXISTS" with the given select
    * statement and inserts it at the current position.</p>
    */
   public void createEXISTS(SelectStatement pStatement);

   /** <p>Creates a JOIN condition matching the given foreign key. In other
    * words, if the foreign key consists of the columns <code>A</code> and
    * <code>B</code> referencing the columns <code>X</code> and <code>Y</code>,
    * then the following will be added: <code>A=X AND B=Y</code>.</p>
    * @param pKey The foreign key being matched.
    * @param pReferencingTable A reference to the table returned by the
    *   foreign keys {@link ForeignKey#getTable()} method.
    * @param pReferencedTable A reference to the table returned by the
    *   foreign keys {@link ForeignKey#getReferencedTable()} method.
    */
   public void addJoin(ForeignKey pKey, TableReference pReferencingTable,
                        TableReference pReferencedTable);

   /** <p>Creates a JOIN condition matching the given column reference.
    * In other words, if the referencing {@link ColumnSet} contains the
    * columns A and B, and the referenced column set contains {@link ColumnSet},
    * X and Y, then the following will be added: <code>A=X AND B=Y</code>.</p>
    */
   public void addJoin(TableReference pReferencingTable,
                       ColumnSet pReferencingColumnSet,
                       TableReference pReferencedTable,
                       ColumnSet pReferencedColumnSet);

   /** <p>Clones the given CombinedConstraint, mapping the column references
    * from the given constraint to the values in the given map.</p>
    * @param pMap A Map with the constraints <code>pConstraint</code> table
    *   references as keys. The values are table references of the current
    *   constraints statement.
    * @param pConstraint The constraint being cloned.
    */
   public void addConstraint(Map pMap, Constraint pConstraint);

   /** <p>Adds a check for the columns of the given column set. For example,
    * if the column set consists of the columns <code>A</code> and <code>B</code>,
    * then the following will be added: <code>A=? AND B=?</code>.</p>
    */
   public void addColumnSetQuery(ColumnSet pSet, TableReference pTableReference);

   /** <p>Returns the number of parts, that have been added with the
    * various <code>createSomething()</code> methods.</p>
    */
   public int getNumParts();

   /** <p>Returns an Iterator to the parts, that have been added with the
    * various <code>createSomething()</code> methods.</p>
    */
   public Iterator getParts();
}
