/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls;

import java.io.Serializable;
import java.util.Iterator;


/** <p>Interface of a foreign key.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface ForeignKey extends ColumnSet {
   public class Mode implements Serializable {
      private String name;
      private Mode(String pName) {
         name = pName;
      }
      public String getName() { return name; }
      public String toString() { return name; }
      public static final Mode CASCADE = new Mode("CASCADE");
      public static final Mode REJECT = new Mode("REJECT");
      public static final Mode SETNULL = new Mode("SETNULL");
      private static final Mode[] instances = new Mode[]{
         CASCADE, REJECT, SETNULL
      };
      public static Mode[] getInstances() {
         return instances;
      }
      public static Mode valueOf(String pMode) {
         for (int i = 0;  i < instances.length;  i++) {
            if (instances[i].getName().equals(pMode)) {
               return instances[i];
            }
         }
         throw new IllegalArgumentException("Unknown mode: " + pMode);
      }
      public int hashCode() {
         return name.hashCode();
      }
      public boolean equals(Object o) {
         return o != null  &&  (o instanceof Mode)  &&  name.equals(((Mode) o).name);
      }
   }

   public interface ColumnLink {
      /** <p>Returns the column referencing a column in the referenced
       * table.</p>
       */
      public Column getLocalColumn();
      /** <p>Returns the column being referenced in the referenced
       * table.</p>
       */
      public Column getReferencedColumn();
   }

   /** <p>Returns the referenced table.</p>
    */
   public Table getReferencedTable();

   /** <p>Sets the OnDelete mode.</p>
    */
   public void setOnDelete(Mode pMode);

   /** <p>Returns the OnDelete mode.</p>
    */
   public Mode getOnDelete();

   /** <p>Sets the OnUpdate mode.</p>
    */
   public void setOnUpdate(Mode pMode);

   /** <p>Returns the OnUpdate mode.</p>
    */
   public Mode getOnUpdate();

   /** <p>Adds a reference between the given columns.</p>
    * @param pColumn A column of the table, on which the foreign key is
    *   defined
    * @param pReferencedColumn A column of the referenced table
    */
   public void addColumnLink(Column pColumn, Column pReferencedColumn);

   /** <p>Adds a reference between the given columns.</p>
    * @param pName Column name of the table, on which the foreign
    *   key is defined
    * @param pReferencedName Column name of the referenced table.
    */
   public void addColumnLink(Column.Name pName, Column.Name pReferencedName);

   /** <p>Adds a reference between the given columns.</p>
    * @param pName Column name of the table, on which the foreign
    *   key is defined
    * @param pReferencedName Column name of the referenced table.
    */
   public void addColumnLink(String pName, String pReferencedName);

   /** <p>Returns all column references in the foreign key. Any instance
    * returned by the {@link Iterator} is an instance of
    * {@link ForeignKey.ColumnLink}.</p>
    */
   public Iterator getColumnLinks();

   /** <p>Returns the set of referenced columns.</p>
    */
   public ColumnSet getReferencedColumns();
}
