/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Iterator;


/** <p>A factory for generating SQL statements.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface SQLFactory {
  public interface Ident {
    /** <p>Returns the SQL identifiers String representation. Typically
     * same than {@link Object#toString()}.</p>
     */
    public String getName();
  }

  /** <p>Returns the object factory being used.</p>
   */
  public ObjectFactory getObjectFactory();

  /** <p>Returns the maximum length of a table name.</p>
   *
   * @return The maximum length or null, if checks for valid table name
   *   length are disabled.
   */
  public Integer getMaxTableNameLength();

  /** <p>Returns whether table names are case sensitive or not. Defaults to
   * false.</p>
   */
  public boolean isTableNameCaseSensitive();

  /** <p>Returns the maximum length of a schema name.</p>
   *
   * @return The maximum length or null, if checks for valid schema name
   *   length are disabled.
   */
  public Integer getMaxSchemaNameLength();

  /** <p>Returns whether schema names are case sensitive or not. Defaults to
   * false.</p>
   */
  public boolean isSchemaNameCaseSensitive();

  /** <p>Returns the maximum length of a column name.</p>
   *
   * @return The maximum length or null, if checks for valid column name
   *   length are disabled.
   */
  public Integer getMaxColumnNameLength();

  /** <p>Returns whether column names are case sensitive or not. Defaults to
   * false.</p>
   */
  public boolean isColumnNameCaseSensitive();

  /** <p>Creates a new SELECT statement.</p>
   */
  public SelectStatement newSelectStatement();

  /** <p>Creates a new INSERT statement.</p>
   */
  public InsertStatement newInsertStatement();

  /** <p>Creates a new UPDATE statement.</p>
   */
  public UpdateStatement newUpdateStatement();

  /** <p>Creates a new DELETE statement.</p>
   */
  public DeleteStatement newDeleteStatement();

  /** <p>Creates a new {@link Schema} with the given name.</p>
   */
  public Schema newSchema(String pName);

  /** <p>Creates a new {@link Schema} with the given name.</p>
   */
  public Schema newSchema(Schema.Name pName);

  /** <p>Returns the {@link Schema Default schema}. The default
   * schema has the name null.</p>
   */
  public Schema getDefaultSchema();

  /** <p>Returns the schema with the given name or null, if no such
   * schema exists.</p>
   */
  public Schema getSchema(Schema.Name pName);

  /** <p>Returns the schema with the given name or null, if no such
   * schema exists.</p>
   */
  public Schema getSchema(String pName);

  /** <p>Returns a list of all schemas. The list includes the default
   * schema, if {@link #getDefaultSchema()} was called at any time.</p>
   */
  public Iterator getSchemas();

  /** <p>Creates a new {@link SQLGenerator}.</p>
   */
  public SQLGenerator newSQLGenerator();

  /** <p>Reads the schema named <code>pName</code> from the database.</p>
   */
  public Schema getSchema(Connection pConnection, Schema.Name pName) throws SQLException;

  /** <p>Reads the schema named <code>pName</code> from the database.</p>
   */
  public Schema getSchema(Connection pConnection, String pName) throws SQLException;

  /** <p>Reads the table named <code>pTable</code> from the schema
   * named <code>pSchema</code> in the database.</p>
   */
  public Table getTable(Connection pConnection, Schema.Name pSchema, Table.Name pTable) throws SQLException;

  /** <p>Reads the table named <code>pTable</code> from the schema
   * named <code>pSchema</code> in the database.</p>
   */
  public Table getTable(Connection pConnection, String pSchema,String pTable) throws SQLException;
}
