/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls;

import java.util.Collection;


/** <p>An SQL generator.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface SQLGenerator {
  /** <p>Generates a <code>CREATE SCHEMA</code> statement. Doesn't create
   * <code>CREATE TABLE</code> or similar statements.</p>
   */
  public Collection getCreate(Schema pSchema);

  /** <p>Generates <code>CREATE</code> statements for the schema.</p>
   * @param pAll If this parameter is set to true, then the method is
   *    equivalent to  {@link #getCreate(Schema)}. Otherwise the returned
   *    collection will also include <code>CREATE</code> statements for
   *    all the tables and indexes in the schema. These additional statements
   *    are created by invoking {@link #getCreate(Table,boolean)} for all the
   *    tables in the schema.</p>
   */
  public Collection getCreate(Schema pSchema, boolean pAll);

  /** <p>Generates a DROP SCHEMA statement. Doesn't create
   * <code>DROP TABLE</code> or similar statements.</p>
   */
  public Collection getDrop(Schema pSchema);

  /** <p>Generates <code>DROP</code> statements for the schema.</p>
   * @param pAll If this parameter is set to true, then the method is
   *    equivalent to  {@link #getDrop(Schema)}. Otherwise the returned
   *    collection will also include <code>DROP</code> statements for
   *    all the tables and indexes in the schema. These additional statements
   *    are created by invoking {@link #getDrop(Table,boolean)} for all the
   *    tables in the schema.</p>
   */
  public Collection getDrop(Schema pSchema, boolean pAll);

  /** <p>Generates a CREATE TABLE statement. Doesn't create
   * <code>CREATE INDEX</code> or similar statements.</p>
   */
  public Collection getCreate(Table pTable);

  /** <p>Generates <code>CREATE</code> statements for the table.</p>
   * @param pAll If this parameter is set to true, then the method is
   *    equivalent to  {@link #getCreate(Table)}. Otherwise the returned
   *    collection will also include <code>CREATE</code> statements for
   *    the indexes, which are defined on the table. These additional
   *    statements are created by invoking {@link #getCreate(Index)}
   *    and {@link #getCreate(ForeignKey)} for all the indexes in the
   *    schema.</p>
   */
  public Collection getCreate(Table pTable, boolean pAll);

  /** <p>Generates a DROP TABLE statement. Doesn't create
   * <code>DROP INDEX</code> or similar statements.</p>
   */
  public Collection getDrop(Table pTable);

  /** <p>Generates <code>DROP</code> statements for the table.</p>
   * @param pAll If this parameter is set to true, then the method is
   *    equivalent to  {@link #getDrop(Table)}. Otherwise the returned
   *    collection will also include <code>DROP</code> statements for
   *    the indexes, which are defined on the table. These additional
   *    statements are created by invoking {@link #getDrop(Index)}
   *    and {@link #getDrop(ForeignKey)} for all the indexes in the
   *    schema.</p>
   */
  public Collection getDrop(Table pTable, boolean pAll);

  /** <p>Generates a CREATE INDEX statement.</p>
   */
  public Collection getCreate(Index pIndex);

  /** <p>Generates a DROP INDEX statement.</p>
   */
  public Collection getDrop(Index pIndex);

  /** <p>Generates a CREATE FOREIGN KEY statement.</p>
   */
  public Collection getCreate(ForeignKey pKey);

  /** <p>Generates a DROP FOEIGN KEY statement.</p>
   */
  public Collection getDrop(ForeignKey pKey);

  /** <p>Generates an INSERT, UPDATE, DELETE or SELECT statement.</p>
   */
  public String getQuery(Statement pStatement);

  /** <p>Generates the WHERE clause of a SELECT, UPDATE, or DELETE statement.</p>
   */
  public String getConstraint(Constraint pConstraint);

  /** <p>Sets the statement terminator. A non-null value will be
   * appended to all generated statements. Defaults to null.</p>
   */
  public void setStatementTerminator(String pTerminator);

  /** <p>Returns the statement terminator. A non-null value will be
   * appended to all generated statements. Defaults to null.</p>
   */
  public String getStatementTerminator();

  /** <p>Sets the line terminator. A non-null value indicates that
   * the generated statements should be made human readable by splitting
   * them over multiple lines. A null value ensures that a statement
   * consists of a single line only. Defaults to "\n".</p>
   */
  public void setLineTerminator(String pTerminator);

  /** <p>Returns the line terminator. A non-null value indicates that
   * the generated statements should be made human readable by splitting
   * them over multiple lines. A null value ensures that a statement
   * consists of a single line only. Defaults to "\n".</p>
   */
  public String getLineTerminator();
}
