/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls;

import java.util.Iterator;


/** <p>Abstract description of a table.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public interface Table {
   public interface Name extends SQLFactory.Ident {
   }

   /** <p>Returns the table name.</p>
    */
   public Name getName();

   /** <p>Returns the table schema.</p>
    */
   public Schema getSchema();

   /** <p>Returns the table columns.</p>
    */
   public Iterator getColumns();

   /** <p>Creates a new column.</p>
    */
   public Column newColumn(Column.Name name, Column.Type pType);

   /** <p>Creates a new column.</p>
    */
   public Column newColumn(String name, Column.Type pType);

   /** <p>Returns the column with the given name or null,
    * if no such column exists.</p>
    */
   public Column getColumn(Column.Name name);

   /** <p>Returns the column with the given name or null,
    * if no such column exists.</p>
    */
   public Column getColumn(String name);

   /** <p>Creates a new, unique index on the table.</p>
    */
   public Index newKey();

   /** <p>Creates a new, non-unique index on the table.</p>
    */
   public Index newIndex();

   /** <p>Creates a new primary key on the table.</p>
    * @throws IllegalStateException A primary key has already been created.
    */
   public Index newPrimaryKey();

   /** <p>Creates a new foreign key referencing the given table.</p>
    */
   public ForeignKey newForeignKey(Table pReferencedTable);

  /** <p>Returns an INSERT statement for filling all the values. In
   * other words: If the table FOO has the columns A, B, and C,
   * then the statement <code>INSERT INTO FOO (A,B,C) VALUES (?, ?, ?)</code>
   * will be returned.</p>
   * @see SQLFactory#newInsertStatement()
   */
  public InsertStatement getInsertStatement();

  /** <p>Returns a SELECT statement for selecting all the columns. In
   * other words: If the table FOO has the columns A, B, and C,
   * then the statement <code>SELECT A, B, C FROM FOO</code> will
   * be returned.</p>
   * @see SQLFactory#newSelectStatement()
   */
  public SelectStatement getSelectStatement();

  /** <p>Returns an UPDATE statement for updating a column in the table.
   * In other words: If the table FOO has the columns A, B, C and D
   * with the primary key columns A and B, then the statement
   * <code>UPDATE FOO SET C = ?, D = ? WHERE A = ? AND B = ?</code>
   * will be returned.</p>
   * @see SQLFactory#newUpdateStatement()
   * @throws IllegalStateException The table doesn't have a primary key.
   */
  public UpdateStatement getUpdateStatement();

  /** <p>Returns an UPDATE statement for updating a column in the table.
   * In other words: If the table FOO has the primary key columns A and B,
   * then the statement <code>DELETE FROM FOO WHERE A = ? AND B = ?</code>
   * will be returned.</p>
   * @see SQLFactory#newDeleteStatement()
   * @throws IllegalStateException The table doesn't have a primary key.
   */
  public DeleteStatement getDeleteStatement();

  /** <p>Returns the tables qualified name, which is
   * <code>getSchema().getName() + "." + getName()</code>. If the
   * schema is the default schema, returns <code>getName()</code>.</p> 
   */
  public String getQName();

  /** <p>Returns the tables primary key, if any, or null, if the table
   * doesn't have a primary key.</p>
   */
  public Index getPrimaryKey();

  /** <p>Returns an {@link Iterator} to the indexes defined on the table.
   * This iterator includes the primary key, if any.</p>
   */
  public Iterator getIndexes();

  /** <p>Returns an {@link Iterator} to the foreign keys defined on the
   * table.</p>
   */
  public Iterator getForeignKeys();
}
