/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls.db2;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.ws.jaxme.sqls.Column;
import org.apache.ws.jaxme.sqls.SQLGenerator;
import org.apache.ws.jaxme.sqls.Schema;
import org.apache.ws.jaxme.sqls.Table;
import org.apache.ws.jaxme.sqls.impl.SQLFactoryImpl;


/** <p>Default implementation of an SQL factory for DB2 databases.
 * This factory ensures that the created implementations of
 * {@link Schema}, {@link Table}, {@link Column}, and {@link SQLGenerator}
 * may be casted to {@link DB2Schema}, {@link DB2Table}, {@link DB2Column},
 * {@link DB2SQLGenerator}, respectively.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class DB2SQLFactoryImpl extends SQLFactoryImpl
    implements DB2SQLFactory {
  /** <p>An immutable, predefined TableSpace.</p>
   */
  public class PredefinedTableSpace implements TableSpace {
    private TableSpace.Name name;
    private TableSpace.Type type;

    PredefinedTableSpace(String pName, TableSpace.Type pType) {
      name = new TableSpaceImpl.NameImpl(pName);
      type = pType;
    }    
    public DB2SQLFactory getSQLFactory() { return DB2SQLFactoryImpl.this; }
    public Name getName() { return name; }
    public Type getType() { return type; }
    public PageSize getPageSize() { return null; }
    public Long getExtentSize() { return null; }
    public Long getPrefetchSize() { return null; }
    public Number getOverhead() { return null; }
    public Number getTransferRate() { return null; }
    public Boolean hasDroppedTableRecovery() { return null; }
    public BufferPool getBufferPool() { return null; }
    public boolean isPredefined() { return true; }
    public void setPageSize(PageSize pSize) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public void setExtentSize(Long pSize) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public void setPrefetchSize(Long pSize) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public void setOverhead(Number pOverhead) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public void setTransferRate(Number pNumber) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public void setDroppedTableRecovery(Boolean pRecoverable) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public Container newSystemManagedContainer(String pFile) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public Container newDatabaseManagedContainerInFile(String pFile, long pNumPages) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public Container newDatabaseManagedContainerInDevice(String pDevice, long pNumPages) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
    public Iterator getContainers() {
      return Collections.EMPTY_LIST.iterator();
    }
    public void setBufferPool(BufferPool pBufferPool) {
      throw new IllegalStateException("This tablespace is immutable.");
    }
  }

  /** <p>The predefined table space <code>SYSCATSPACE</code>.</p>
   */
  public final TableSpace SYSCATSPACE = new PredefinedTableSpace("SYSCATSPACE", TableSpace.Type.REGULAR);
  /** <p>The predefined table space <code>TEMPSPACE1</code>.</p>
   */
  public final TableSpace TEMPSPACE1 = new PredefinedTableSpace("TEMPSPACE1", TableSpace.Type.SYSTEM_TEMPORARY);
  /** <p>The predefined table space <code>USERSPACE1</code>.</p>
   */
  public final TableSpace USERSPACE1 = new PredefinedTableSpace("USERSPACE1", TableSpace.Type.USER_TEMPORARY);

  private List tableSpaces = new ArrayList();

  public DB2SQLFactoryImpl() {}

  public Schema newSchemaImpl(Schema.Name pName) {
    return new DB2SchemaImpl(this, pName);
  }

  public Table newTableImpl(Schema pSchema, Table.Name pName) {
    return new DB2TableImpl(pSchema, pName);
  }

  public Column newColumn(Table pTable, Column.Name pName, Column.Type pType) {
    return new DB2ColumnImpl(pTable, pName, pType);
  }

  public SQLGenerator newSQLGenerator() {
    return new DB2SQLGeneratorImpl();
  }

  public TableSpace newTableSpace(String pName, TableSpace.Type pType) {
    if (pName == null) {
      throw new NullPointerException("The tablespace name must not be null. Use getDefaultTableSpace() to access the default TableSpace."); 
    }
    return newTableSpace(new TableSpaceImpl.NameImpl(pName), pType);
  }

  public TableSpace newTableSpace(TableSpace.Name pName, TableSpace.Type pType) {
    if (pName == null) {
      throw new NullPointerException("The tablespace name must not be null. Use getDefaultTableSpace() to access the default TableSpace."); 
    }
    if (getTableSpace(pName) != null) {
      throw new IllegalStateException("A TableSpace named " + pName + " already exists.");
    }
    TableSpace result = newTableSpaceImpl(pName, pType);
    tableSpaces.add(result);
    return result;
  }

  protected TableSpace newTableSpaceImpl(TableSpace.Name pName, TableSpace.Type pType) {
    return new TableSpaceImpl(this, pName, pType);
  }

  public TableSpace getTableSpace(TableSpace.Name pName) {
    for (Iterator iter = tableSpaces.iterator();  iter.hasNext();  ) {
      TableSpace tableSpace = (TableSpace) iter.next();
      if (pName.equals(tableSpace.getName())) {
        return tableSpace;
      }
    }
    if (pName.equals(SYSCATSPACE.getName())) {
       return SYSCATSPACE;
    }
    if (pName.equals(TEMPSPACE1.getName())) {
       return TEMPSPACE1;
    }
    if (pName.equals(USERSPACE1.getName())) {
       return USERSPACE1;
    }
    return null;
  }

  public TableSpace getTableSpace(String pName) {
    return getTableSpace(new TableSpaceImpl.NameImpl(pName)); 
  }

  public Iterator getTableSpaces() {
    return tableSpaces.iterator();
  }
}
