/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.ws.jaxme.sqls.Column;
import org.apache.ws.jaxme.sqls.ColumnSet;
import org.apache.ws.jaxme.sqls.ForeignKey;
import org.apache.ws.jaxme.sqls.Table;


/** <p>Implementation of a foreign key.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class ForeignKeyImpl extends ColumnSetImpl implements ForeignKey {
  public static class ColumnReferenceImpl implements ForeignKey.ColumnLink {
    private final Column localColumn, referencedColumn; 
    public ColumnReferenceImpl(Column pLocalColumn, Column pReferencedColumn) {
      if (pLocalColumn == null) {
         throw new NullPointerException("The local column must not be null.");
      }
      if (pReferencedColumn == null) {
        throw new NullPointerException("The referenced column must not be null.");
      }
      localColumn = pLocalColumn;
      referencedColumn = pReferencedColumn;
    }
    public Column getLocalColumn() {
      return localColumn;
    }
    public Column getReferencedColumn() {
      return referencedColumn;
    }
  }

  private Table referencedTable;
  private ForeignKey.Mode onDelete, onUpdate;
  public List references = new ArrayList();

  protected ForeignKeyImpl(Table pTable, Table pReferencedTable) {
    super(pTable);
    if (pReferencedTable == null) {
      throw new NullPointerException("The referenced table must not be null.");
    }
    referencedTable = pReferencedTable;
  }

	public Table getReferencedTable() {
      return referencedTable;
	}

	public void setOnDelete(ForeignKey.Mode pMode) {
      onDelete = pMode;
	}

	public ForeignKey.Mode getOnDelete() {
      return onDelete;
	}

	public void setOnUpdate(ForeignKey.Mode pMode) {
      onUpdate = pMode;
	}

	public ForeignKey.Mode getOnUpdate() {
      return onUpdate;
	}

	public void addColumnLink(Column pColumn, Column pReferencedColumn) {
    addColumnReference(new ColumnReferenceImpl(pColumn, pReferencedColumn));
  }

  public void addColumnReference(ForeignKey.ColumnLink pReference) {
    Column localColumn = pReference.getLocalColumn();
    if (!getTable().equals(localColumn.getTable())) {
      throw new IllegalStateException("The local column " + localColumn.getQName() +
                                       " is not from the local table " + getTable().getQName());
    }
    Column referencedColumn = pReference.getReferencedColumn();
    if (!getReferencedTable().equals(referencedColumn.getTable())) {
      throw new IllegalStateException("The referenced column " + referencedColumn.getQName() +
                                       " is not from the referenced table " + getReferencedTable().getQName());
    }
    references.add(pReference);
	}

	public void addColumnLink(Column.Name pName, Column.Name pReferencedName) {
      Column localColumn = getTable().getColumn(pName);
      if (localColumn == null) {
        throw new NullPointerException("The local table " + getTable().getQName() +
                                        " doesn't contain a column " + pName);
      }
      Column referencedColumn = getReferencedTable().getColumn(pReferencedName);
      if (referencedColumn == null) {
         throw new NullPointerException("The referenced table " + getReferencedTable().getName() +
                                         " doesn't contain a column " + pReferencedName);
      }
      addColumnLink(localColumn, referencedColumn);
	}

	public void addColumnLink(String pName, String pReferencedName) {
      addColumnLink(new ColumnImpl.NameImpl(pName), new ColumnImpl.NameImpl(pReferencedName));
	}

   public Iterator getColumnLinks() {
      return references.iterator();
   }

	public Iterator getColumns() {
      return new Iterator(){
         Iterator inner = getColumnLinks();
         public boolean hasNext() { return inner.hasNext(); }
         public void remove() { inner.remove(); }
         public Object next() {
            ForeignKey.ColumnLink link = (ForeignKey.ColumnLink) inner.next();
            return link.getLocalColumn();
         }
      };
	}

   public ColumnSet getReferencedColumns() {
     return new ColumnSet(){
       public Table getTable() { return getReferencedTable(); }
       public Iterator getColumns() {
         return new Iterator() {
            private Iterator inner = getColumnLinks();
            public Object next() {
              return ((ColumnLink) inner.next()).getReferencedColumn();
            }
            public boolean hasNext() { return inner.hasNext(); }
            public void remove() { inner.remove(); }
         };
       }
     };
   }
}
