/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.ws.jaxme.sqls.ColumnReference;
import org.apache.ws.jaxme.sqls.Function;
import org.apache.ws.jaxme.sqls.SelectStatement;
import org.apache.ws.jaxme.sqls.Statement;
import org.apache.ws.jaxme.sqls.Value;

/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public abstract class PartsImpl {
  private final Statement statement;
  private final List parts = new ArrayList();

  protected PartsImpl(Statement pStatement) {
    statement = pStatement;
  }

  protected void add(Object o) {
    parts.add(o);
  }

  public Statement getStatement() {
    return statement;
  }

  public void addPart(Value pValue) {
    if (pValue == null) {
      throw new NullPointerException("A constant value must not be null.");
    }
    parts.add(pValue);
  }

  public void addPart(ColumnReference pColumn) {
    if (pColumn == null) {
      throw new NullPointerException("Referenced column must not be null.");
    }
    add(pColumn);
  }

  public void addPart(ColumnReference[] pColumns) {
    if (pColumns == null) {
      throw new NullPointerException("The array of referenced columns must not be null.");
    }
    for (int i = 0;  i < pColumns.length;  i++) {
      if (pColumns[i] == null) {
        throw new NullPointerException("The referenced column with number " + i + " must not be null.");
      }
    }
    add(pColumns);
  }

  public void addPart(SelectStatement pStatement) {
    if (pStatement == null) {
      throw new NullPointerException("The subselect statement must not be null.");
    }
    add(pStatement);
  }

  public void addPart(String pString) {
    add(new ValueImpl(Value.Type.STRING, pString));
  }

  public void addPart() {
    add(new ValueImpl(Value.Type.NULL, null));
  }

  public void addPart(byte pByte) {
    add(new ValueImpl(Value.Type.BYTE, new Byte(pByte)));
  }

  public void addPart(int pInt) {
    add(new ValueImpl(Value.Type.INT, new Integer(pInt)));
  }

  public void addPart(long pLong) {
    add(new ValueImpl(Value.Type.LONG, new Long(pLong)));
  }

  public void addPart(short pShort) {
    add(new ValueImpl(Value.Type.SHORT, new Short(pShort)));
  }

  public void addPart(float pFloat) {
    add(new ValueImpl(Value.Type.FLOAT, new Float(pFloat)));
  }

  public void addPart(double pDouble) {
    add(new ValueImpl(Value.Type.DOUBLE, new Double(pDouble)));
  }

  public void addPart(boolean pBoolean) {
    add(new ValueImpl(Value.Type.BOOLEAN, pBoolean ? Boolean.TRUE : Boolean.FALSE));
  }

  public void addPart(Function pFunction) {
    add(pFunction);
  }

  public void addPlaceholder() {
    add(new ValueImpl(Value.Type.PLACEHOLDER, null));
  }

  /** <p>Inserts raw SQL code.</p>
   */ 
  public void addRawSQLPart(String pRawSQL) {
    add(getStatement().getSQLFactory().getObjectFactory().newRawSQL(pRawSQL));
  }

  public int getNumParts() {
    return parts.size();
  }

  public Iterator getParts() {
    return parts.iterator();
  }
}
