/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.sqls.oracle;

import java.sql.DatabaseMetaData;
import java.sql.Types;

import org.apache.ws.jaxme.logging.Logger;
import org.apache.ws.jaxme.logging.LoggerAccess;
import org.apache.ws.jaxme.sqls.BinaryColumn;
import org.apache.ws.jaxme.sqls.Column;
import org.apache.ws.jaxme.sqls.ObjectFactory;
import org.apache.ws.jaxme.sqls.SQLGenerator;
import org.apache.ws.jaxme.sqls.StringColumn;
import org.apache.ws.jaxme.sqls.Table;
import org.apache.ws.jaxme.sqls.impl.SQLFactoryImpl;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class OraSQLFactoryImpl extends SQLFactoryImpl implements OraSQLFactory {
  private static final Logger logger = LoggerAccess.getLogger(SQLFactoryImpl.class);

  public SQLGenerator newSQLGenerator() {
    return new OraSQLGeneratorImpl();
  }

  protected Column readColumn(Table pTable, String pColumnName, int pDataType, String pTypeName,
                               long pColumnSize, int pDecimalDigits, int pNullable) {
                               Column.Type type;
    final String mName = "readColumn";
    if ("NUMBER".equalsIgnoreCase(pTypeName)) {
      pDataType = Types.NUMERIC;
    }
    switch (pDataType) {
      case Types.BIGINT:        type = Column.Type.BIGINT;    break;
      case Types.BINARY:        type = Column.Type.BINARY;    break;
      case Types.BIT:           type = Column.Type.BIT;       break;
      case Types.BLOB:          type = Column.Type.VARBINARY; break;
      case Types.BOOLEAN:       type = Column.Type.BIT;       break;
      case Types.CHAR:          type = Column.Type.CHAR;      break;
      case Types.CLOB:          type = Column.Type.VARCHAR;   break;
      case Types.DATE:          type = Column.Type.DATE;      break;
      case Types.DOUBLE:        type = Column.Type.DOUBLE;    break;
      case Types.FLOAT:         type = Column.Type.FLOAT;     break;
      case Types.INTEGER:       type = Column.Type.INTEGER;   break;
      case Types.LONGVARBINARY: type = Column.Type.VARBINARY; break;
      case Types.LONGVARCHAR:   type = Column.Type.VARCHAR;   break;
      case Types.NUMERIC:
        if (pDecimalDigits == 0) {
          if (pColumnSize == 0) {
            type = Column.Type.FLOAT;
          } else if (pColumnSize <= 2) {
            type = Column.Type.TINYINT;
          } else if (pColumnSize <= 4) {
            type = Column.Type.SMALLINT;
          } else if (pColumnSize <= 9) {
            type = Column.Type.INTEGER;
          } else {
            type = Column.Type.BIGINT;
          }
        } else if (pDecimalDigits == -127) {
          // Uses binary precision - See page 4-37 of the OCI book
          if (pColumnSize < 24) {
            type = Column.Type.FLOAT;  // 53 is double cutoff
          } else {
            type = Column.Type.DOUBLE;
          }
        } else {
          // Uses decimal precision - See page 4-37 of the OCI book
          if (pColumnSize < 8) {
            type = Column.Type.FLOAT;   // 15 is double cutoff
          } else {
            type = Column.Type.DOUBLE;
          }
        }
        break;
      case Types.OTHER:
        if ("CLOB".equalsIgnoreCase(pTypeName)) {
          type = Column.Type.CLOB;
        } else {
          type = Column.Type.BLOB;
        }
        break;        
      case Types.SMALLINT:      type = Column.Type.SMALLINT;  break;
      case Types.TIMESTAMP:     type = Column.Type.TIMESTAMP; break;
      case Types.TIME:          type = Column.Type.TIME;      break;
      case Types.TINYINT:       type = Column.Type.TINYINT;   break;
      case Types.VARBINARY:     type = Column.Type.VARBINARY; break;
      case Types.VARCHAR:       type = Column.Type.VARCHAR;   break;
      default: throw new IllegalArgumentException("Column " + pColumnName +
                                                    " in table " + pTable.getQName() +
                                                    " has unknown JDBC data type " +
                                                    pDataType);
    }
    Column column = pTable.newColumn(pColumnName, type);
    logger.finest(mName, "Found column " + pColumnName);
    if (column instanceof StringColumn) {
      ((StringColumn) column).setLength(pColumnSize);
    } else if (column instanceof BinaryColumn) {
      ((BinaryColumn) column).setLength(pColumnSize);
    }
    if (pNullable == DatabaseMetaData.columnNullable) {
      column.setNullable(true);
    }
    return column;
  }

  protected ObjectFactory newObjectFactory() {
    return new OraObjectFactoryImpl();
  }
}
