/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.pm.ino;

import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;

import javax.xml.bind.Element;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.apache.ws.jaxme.JMHandler;
import org.apache.ws.jaxme.JMManager;
import org.apache.ws.jaxme.JMUnmarshaller;
import org.apache.ws.jaxme.Observer;
import org.apache.ws.jaxme.PMException;
import org.apache.ws.jaxme.PMParams;
import org.apache.ws.jaxme.pm.impl.PMIdImpl;



/** <p>An implementation of a JMManager for a Tamino database.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>s
 */
public class InoManager extends PMIdImpl {
  private java.net.URL dbURL;
  private String user, password;
  private String idTag, elementTag;
  private boolean useGet;
  private static final javax.xml.parsers.SAXParserFactory spf;

  static {
    spf = SAXParserFactory.newInstance();
    spf.setNamespaceAware(true);
    spf.setValidating(false);
  }

  public String getElementTag() {
    return elementTag;
  }

  public void setElementTag(String pElementTag) {
    elementTag = pElementTag;
  }

  public String getIdTag() {
    return idTag;
  }

  public void setIdTag(String pIdTag) {
    idTag = pIdTag;
  }

  public void init(JMManager pManager) throws JAXBException {
    super.init(pManager);
    idTag = pManager.getProperty("ino.idTag");
    if (idTag == null  ||  idTag.length() == 0) {
      throw new JAXBException("Missing property: 'ino.idTag' (Tag name or attribute name of the element ID)");
    }
    elementTag = pManager.getProperty("ino.elementTag");
    if (elementTag == null  ||  elementTag.length() == 0) {
      throw new JAXBException("Missing property: 'ino.elementTag' (Qualified element name, including namespace prefix)");
    }
    String url = pManager.getProperty("ino.url");
    if (url == null  ||  url.length() == 0) {
      throw new JAXBException("Missing property: 'ino.url' (Tamino database URL)");
    }
    user = pManager.getProperty("ino.user");
    password = pManager.getProperty("ino.password");
    useGet = Boolean.valueOf(pManager.getProperty("ino.useGet")).booleanValue();
  }

  /** <p>Returns a query suited for deleting the element.</p>
   */
  protected String getDeleteQuery(Element pElement)
      throws JAXBException, InvocationTargetException, IllegalAccessException, NoSuchMethodException {
    String id = getId(pElement);
    if (id == null  ||  id.length() == 0) {
      throw new JAXBException("The element being deleted doesn't have an ID.");
    }
    return "_delete=" +
      java.net.URLEncoder.encode(getElementTag() + '[' + getIdTag() + '=' + id + ']');
  }

  /** <p>Returns a query suited for updating the element.</p>
   */
  protected String getUpdateQuery(Element pElement) throws JAXBException {
    StringWriter sw = new StringWriter();
    Marshaller marshaller = getManager().getFactory().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.FALSE);
    marshaller.marshal(pElement, sw);
    return "_process=" + java.net.URLEncoder.encode(sw.toString());
  }

  /** <p>Returns a query suited for inserting the element.</p>
   */
  protected String getInsertQuery(Element pElement) throws JAXBException {
    StringWriter sw = new StringWriter();
    Marshaller marshaller = getManager().getFactory().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.FALSE);
    marshaller.marshal(pElement, sw);
    return "_process=" + java.net.URLEncoder.encode(sw.toString());
  }

  /** <p>Performs a single database query.</p>
   */
  protected java.net.HttpURLConnection getResponse(String pQuery) throws SAXException {
    java.net.URL connectionURL = dbURL;
    try {
      if (useGet) {
        String dburl = connectionURL.toString();
        String url;
        if (dburl.indexOf('?') > 0) {
          url = dburl + "&" + pQuery;
        } else {
          url = dburl + "?" + pQuery;
        }
        try {
          connectionURL = new java.net.URL(url);
          java.net.HttpURLConnection conn =
            (java.net.HttpURLConnection) connectionURL.openConnection();
          conn.setDoOutput(false);
          conn.setDoInput(true);
          return conn;
        } catch (java.net.MalformedURLException e) {
          throw new SAXException("Malformed database URL: " + url);
        }
      } else {
        java.net.HttpURLConnection conn =
          (java.net.HttpURLConnection) connectionURL.openConnection();
        conn.setDoOutput(true);
        conn.setDoInput(true);

        java.io.OutputStream ostream = conn.getOutputStream();
        java.io.Writer w = new java.io.OutputStreamWriter(ostream);
        w.write(pQuery);
        w.close();
        return conn;
      }
    } catch (java.io.IOException e) {
      throw new SAXException("I/O Error: " + e.getMessage(), e);
    }
  }

  /** <p>Performs a single database query.</p>
   */
  protected InoResponseHandler performQuery(String pQuery, java.util.List pList)
      throws SAXException {
    InoResponseHandler irh = new InoResponseHandler();
    if (pList != null) {
      irh.setInoObjectIdList(pList);
    }
    performQuery(pQuery, irh);
    return irh;
  }

  /** <p>Parses a single INO response document.</p>
   */
  protected void performQuery(String pQuery, InoResponseHandler pHandler)
      throws SAXException {
    java.net.HttpURLConnection conn = getResponse(pQuery);
    try {
      org.xml.sax.InputSource isource = new org.xml.sax.InputSource(conn.getInputStream());
      isource.setEncoding(conn.getContentEncoding());
      org.xml.sax.XMLReader xr;
      javax.xml.parsers.SAXParser sp = spf.newSAXParser();
      xr = sp.getXMLReader();
      xr.setContentHandler(pHandler);
      xr.parse(isource);
    } catch (javax.xml.parsers.ParserConfigurationException e) {
      throw new SAXException("ParserConfigurationException: " + e.getMessage(), e);
    } catch (java.io.IOException e) {
      throw new SAXException("I/O Exception: " + e.getMessage(), e);
    }
  }

  /* (non-Javadoc)
   * @see org.apache.ws.jaxme.PM#select(org.apache.ws.jaxme.Observer, java.lang.String, org.apache.ws.jaxme.PMParams)
   */
  public void select(Observer pObserver, String pQuery, PMParams pPlaceHolderArgs) throws PMException {
    pQuery = super.parseQuery(pQuery, pPlaceHolderArgs);
    String q;
    int max = pPlaceHolderArgs.getMaxResultDocuments();
    int skip = pPlaceHolderArgs.getSkippedResultDocuments();
    if (max != 0  ||  skip != 0) {
      q = "_xql(" + (skip+1) + "," + max + ")=";
    } else {
      q = "_xql=";
    }
    q += java.net.URLEncoder.encode(pQuery);

    InoResponseHandler irh = new InoResponseHandler();
    Class c = getManager().getHandlerClass();
    try {
      JMHandler handler = (JMHandler) c.newInstance();
      handler.init((JMUnmarshaller) null);
      handler.setObserver(pObserver);
      irh.setResultHandler(handler);
      performQuery(q, irh);
    } catch (InstantiationException e) {
      throw new PMException(e);
    } catch (IllegalAccessException e) {
      throw new PMException(e);
    } catch (SAXException e) {
      throw new PMException(e);
    } catch (JAXBException e) {
      if (e instanceof PMException) {
        throw (PMException) e;
      } else {
        throw new PMException(e);
      }
    }
  }

  /* (non-Javadoc)
   * @see org.apache.ws.jaxme.PM#insert(javax.xml.bind.Element)
   */
  public void insert(javax.xml.bind.Element pElement) throws PMException {
    try {
      String query = getInsertQuery(pElement);
      java.util.List idList = new java.util.ArrayList();
      performQuery(query, idList);
      if (idList.size() == 0) {
        throw new PMException("Query did not return an ino:id");
      }
    } catch (SAXException e) {
      throw new PMException(e);
    } catch (JAXBException e) {
      if (e instanceof PMException) {
        throw (PMException) e;
      } else {
        throw new PMException(e);
      }
    }
  }

  /* (non-Javadoc)
   * @see org.apache.ws.jaxme.PM#update(javax.xml.bind.Element)
   */
  public void update(javax.xml.bind.Element pElement) throws PMException {
    try {
      String query = getUpdateQuery(pElement);
      performQuery(query, (java.util.List) null);
    } catch (SAXException e) {
      throw new PMException(e);
    } catch (JAXBException e) {
      if (e instanceof PMException) {
        throw (PMException) e;
      } else {
        throw new PMException(e);
      }
    }
  }

  /* (non-Javadoc)
   * @see org.apache.ws.jaxme.PM#delete(javax.xml.bind.Element)
   */
  public void delete(javax.xml.bind.Element pElement) throws PMException {
    try {
      String query = getDeleteQuery(pElement);
      performQuery(query, (java.util.List) null);
    } catch (NoSuchMethodException e) {
      throw new PMException(e);
    } catch (IllegalAccessException e) {
      throw new PMException(e);
    } catch (InvocationTargetException e) {
      throw new PMException(e.getTargetException());
    } catch (SAXException e) {
      throw new PMException(e);
    } catch (JAXBException e) {
      if (e instanceof PMException) {
        throw (PMException) e;
      } else {
        throw new PMException(e);
      }
    }
  }
}
