/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.impl;

import javax.xml.XMLConstants;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** <p>Serializes a DOM node into a stream of SAX events. This code
 * is duplicated and copied from
 * {@link org.apache.ws.jaxme.util.DOMSerializer}. The reason for
 * duplicating the code is, that the JaxMe runtime should not depend
 * on JaxMeXS. On the other hand, JaxMeXS should not depend on
 * JaxMe.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: DOMSerializer.java 231641 2003-11-10 21:16:13Z jochen $
 */
public class DOMSerializer {
  private boolean namespaceDeclarationAttribute;
  private boolean parentsNamespaceDeclarationDisabled;

  /** <p>Sets whether XML declarations are being serialized as
   * attributes or as SAX events (default).</p>
   */
  public void setNamespaceDeclarationAttribute(boolean pXmlDeclarationAttribute) {
    namespaceDeclarationAttribute = pXmlDeclarationAttribute;
  }

  /** <p>Returns whether XML declarations are being serialized as
   * attributes or as SAX events (default).</p>
   */
  public boolean isNamespaceDeclarationAttribute() {
    return namespaceDeclarationAttribute;
  }

  /** <p>Returns whether XML declarations present in the parent nodes
   * are being serialized (default) or not. This option takes effect
   * only if the namespace declarations are sent as events. In other
   * words, if the <code>namespaceDeclarationAttribute</code>
   * properts is false.</p>
   */
  public void setParentsNamespaceDeclarationDisabled(boolean pParentsXmlDeclarationDisabled) {
    parentsNamespaceDeclarationDisabled = pParentsXmlDeclarationDisabled;
  }

  /** <p>Sets whether XML declarations present in the parent nodes
   * are being serialized (default) or not. This option takes effect
   * only if the namespace declarations are sent as events. In other
   * words, if the <code>namespaceDeclarationAttribute</code>
   * properts is false.</p>
   */
  public boolean isParentsNamespaceDeclarationDisabled() {
    return parentsNamespaceDeclarationDisabled;
  }

  protected void doSerializeChilds(Node pNode, ContentHandler pHandler)
      throws SAXException {
    for (Node child = pNode.getFirstChild();  child != null;
          child = child.getNextSibling()) {
      doSerialize(child, pHandler);
    }
  }

  protected void parentsStartPrefixMappingEvents(Node pNode, ContentHandler pHandler)
      throws SAXException {
    if (pNode != null) {
      parentsStartPrefixMappingEvents(pNode.getParentNode(), pHandler);
      if (pNode.getNodeType() == Node.ELEMENT_NODE) {
        startPrefixMappingEvents(pNode, pHandler);
      }
    }
  }

  protected void parentsEndPrefixMappingEvents(Node pNode, ContentHandler pHandler)
      throws SAXException {
    if (pNode != null) {
      if (pNode.getNodeType() == Node.ELEMENT_NODE) {
        endPrefixMappingEvents(pNode, pHandler);
      }
      parentsEndPrefixMappingEvents(pNode.getParentNode(), pHandler);
    }
  }

  protected void startPrefixMappingEvents(Node pNode, ContentHandler pHandler)
      throws SAXException {
    NamedNodeMap nnm = pNode.getAttributes();
    if (nnm != null) {
      for (int i = 0;  i < nnm.getLength();  i++) {
        Node attr = nnm.item(i);
        if (XMLConstants.XMLNS_ATTRIBUTE_NS_URI.equals(attr.getNamespaceURI())) {
          String prefix;
          if (XMLConstants.XMLNS_ATTRIBUTE.equals(attr.getPrefix())) {
            prefix = attr.getLocalName();
          } else if (XMLConstants.XMLNS_ATTRIBUTE.equals(attr.getNodeName())) {
            prefix = "";
          } else {
            throw new IllegalStateException("Unable to parse namespace declaration: " + attr.getNodeName());
          }
          String uri = attr.getNodeValue();
          if (uri == null) {
            uri = "";
          }
          pHandler.startPrefixMapping(prefix, uri);
        }
      }
    }
  }

  protected void endPrefixMappingEvents(Node pNode, ContentHandler pHandler)
      throws SAXException {
    NamedNodeMap nnm = pNode.getAttributes();
    if (nnm != null) {
      for (int i = nnm.getLength()-1;  i >= 0;  i--) {
        Node attr = nnm.item(i);
        if (XMLConstants.XMLNS_ATTRIBUTE_NS_URI.equals(attr.getNamespaceURI())) {
          String prefix = attr.getLocalName();
          pHandler.endPrefixMapping(prefix);
        }
      }
    }
  }

  public void serialize(Node pNode, ContentHandler pHandler)
      throws SAXException {
    pHandler.startDocument();
    if (!isNamespaceDeclarationAttribute()  &&
        !isParentsNamespaceDeclarationDisabled()) {
      parentsStartPrefixMappingEvents(pNode.getParentNode(), pHandler);
    }
    doSerialize(pNode, pHandler);
    if (!isNamespaceDeclarationAttribute()  &&
        !isParentsNamespaceDeclarationDisabled()) {
      parentsEndPrefixMappingEvents(pNode.getParentNode(), pHandler);
    }
    pHandler.endDocument();
  }

  protected void doSerialize(Node pNode, ContentHandler pHandler)
      throws SAXException {
    switch (pNode.getNodeType()) {
      case Node.DOCUMENT_NODE:
        doSerializeChilds(pNode, pHandler);
        break;
      case Node.DOCUMENT_FRAGMENT_NODE:
        doSerializeChilds(pNode, pHandler);
        break;
      case Node.ELEMENT_NODE:
        AttributesImpl attr = new AttributesImpl();
        boolean isNamespaceDeclarationAttribute = isNamespaceDeclarationAttribute();
        if (!isNamespaceDeclarationAttribute) {
          startPrefixMappingEvents(pNode, pHandler);
        }
        NamedNodeMap nnm = pNode.getAttributes();
        if (nnm != null) {
          for (int i = 0;  i < nnm.getLength();  i++) {
            Node a = nnm.item(i);
            if (isNamespaceDeclarationAttribute  ||
                !XMLConstants.XMLNS_ATTRIBUTE_NS_URI.equals(a.getNamespaceURI())) {
              attr.addAttribute(a.getNamespaceURI(), a.getNodeName(),
                                a.getLocalName(), "CDATA", a.getNodeValue());
            }
          }
        }
        pHandler.startElement(pNode.getNamespaceURI(), pNode.getLocalName(),
                              pNode.getNodeName(), attr);
        doSerializeChilds(pNode, pHandler);
        pHandler.endElement(pNode.getNamespaceURI(), pNode.getLocalName(),
                            pNode.getNodeName());
        if (!isNamespaceDeclarationAttribute) {
          endPrefixMappingEvents(pNode, pHandler);
        }
        break;
      case Node.TEXT_NODE:
      case Node.CDATA_SECTION_NODE:
        String s = pNode.getNodeValue();
        pHandler.characters(s.toCharArray(), 0, s.length());
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
        pHandler.processingInstruction(pNode.getNodeName(), pNode.getNodeValue());
        break;
      case Node.ENTITY_REFERENCE_NODE:
        pHandler.skippedEntity(pNode.getNodeName());
        break;
    }
  }
}
