/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.ws.jaxme.xs.XSModelGroup;
import org.apache.ws.jaxme.xs.XSParticle;
import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XSModelGroupImpl implements XSModelGroup {
  private final XSModelGroup.Compositor compositor;
  private final List particles = new ArrayList();
  private final Locator locator;

  public XSModelGroupImpl(XSModelGroup.Compositor pCompositor, Locator pLocator) {
    if (pCompositor == null) {
      throw new NullPointerException("The model group compositor must not be null.");
    }
    compositor = pCompositor;
    locator = pLocator;
  }

  public Compositor getCompositor() {
    return compositor;
  }

  public boolean isSequence() {
    return XSModelGroup.SEQUENCE.equals(compositor);
  }

  public boolean isChoice() {
    return XSModelGroup.CHOICE.equals(compositor);
  }

  public boolean isAll() {
    return XSModelGroup.ALL.equals(compositor);
  }

  public void addParticle(XSParticle pParticle) throws SAXException {
    if (isAll()) {
      if (pParticle.getMaxOccurs() == -1  ||  pParticle.getMaxOccurs() > 1) {
        throw new LocSAXException("Illegal 'maxOccurs' value inside 'all' group: " + pParticle.getMaxOccurs(),
                                     pParticle.getLocator());
      }
    }
    if (pParticle.getMaxOccurs() != -1  &&  pParticle.getMaxOccurs() < pParticle.getMinOccurs()) {
      throw new LocSAXException("Illegal 'maxOccurs' value, which is lower than 'minOccurs' value: " +
                                   pParticle.getMaxOccurs() + " < " + pParticle.getMinOccurs(),
                                   pParticle.getLocator());
    }
    particles.add(pParticle);
  }

  public XSParticle[] getParticles() {
    return (XSParticle[]) particles.toArray(new XSParticle[particles.size()]);
  }

  public void validate() throws SAXException {
    if (isChoice()  &&  particles.size() == 0) {
      throw new LocSAXException("A 'choice' model group must have at least one 'group', 'any', or 'element'.",
                                   getLocator());
    }
    for (Iterator iter = particles.iterator();  iter.hasNext();  ) {
      XSParticle particle = (XSParticle) iter.next();
      if (particle.isElement()) {
        particle.getElement().validate();
      } else if (particle.isGroup()) {
        particle.getGroup().validate();
      } else if (particle.isWildcard()) {
        particle.getWildcard().validate();
      } else {
        throw new IllegalStateException("Invalid particle");
      }
    }
  }

  public Locator getLocator() {
    return locator;
  }
}
