/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.impl;

import org.apache.ws.jaxme.xs.XSAny;
import org.apache.ws.jaxme.xs.XSElement;
import org.apache.ws.jaxme.xs.XSGroup;
import org.apache.ws.jaxme.xs.XSParticle;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/** <p>Default implementation of a particle.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XSParticleImpl implements XSParticle {
  private final XSGroup group;
  private final XSAny wildcard;
  private final XSElement element;
  private int minOccurs = 1;
  private int maxOccurs = 1;

  public XSParticleImpl(XSGroup pGroup) throws SAXException {
    if (pGroup == null) {
      throw new NullPointerException("The particle group must not be null.");
    }
    group = pGroup;
    element = null;
    wildcard = null;
  }

  public XSParticleImpl(XSAny pWildcard) {
    if (pWildcard == null) {
      throw new NullPointerException("The particle wildcard must not be null.");
    }
    wildcard = pWildcard;
    group = null;
    element = null;
  }

  public XSParticleImpl(XSElement pElement) {
    if (pElement == null) {
      throw new NullPointerException("The particle element must not be null.");
    }
    element = pElement;
    group = null;
    wildcard = null;
  }

  public XSParticle.Type getType() {
    if (group != null) {
      return XSParticle.GROUP;
    } else if (wildcard != null) {
      return XSParticle.WILDCARD;
    } else if (element != null) {
      return XSParticle.ELEMENT;
    } else {
      throw new IllegalStateException("Neither of the particle group, wildcard, or element has been set.");
    }
  }

  public boolean isGroup() {
    return group != null;
  }

  public XSGroup getGroup() {
    if (group == null) {
      throw new IllegalStateException("This particle doesn't have the group type.");
    }
    return group;
  }

  public boolean isWildcard() {
    return wildcard != null;
  }

  public XSAny getWildcard() {
    if (wildcard == null) {
      throw new IllegalStateException("This particle doesn't have the wildcard type.");
    }
    return wildcard;
  }

  public boolean isElement() {
    return element != null;
  }

  public XSElement getElement() {
    if (element == null) {
      throw new IllegalStateException("This particle doesn't have the element type.");
    }
    return element;
  }

  public int getMinOccurs() {
    return minOccurs;
  }

  public void setMinOccurs(int pMinOccurs) {
    minOccurs = pMinOccurs;
  }

  public int getMaxOccurs() {
    return maxOccurs;
  }

  public void setMaxOccurs(int pMaxOccurs) {
    maxOccurs = pMaxOccurs;
  }

  public Locator getLocator() {
    if (isWildcard()) {
      return getWildcard().getLocator();
    } else if (isElement()) {
      return getElement().getLocator();
    } else if (isGroup()) {
      return getGroup().getLocator();
    } else {
      throw new IllegalStateException("Invalid particle, neither of element, wildcard, or model group.");
    }
  }
}
