/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.NoSuchElementException;

import org.apache.ws.jaxme.xs.XSAnnotation;
import org.apache.ws.jaxme.xs.XSAttribute;
import org.apache.ws.jaxme.xs.XSAttributeGroup;
import org.apache.ws.jaxme.xs.XSElement;
import org.apache.ws.jaxme.xs.XSGroup;
import org.apache.ws.jaxme.xs.XSNotation;
import org.apache.ws.jaxme.xs.XSObject;
import org.apache.ws.jaxme.xs.XSObjectFactory;
import org.apache.ws.jaxme.xs.XSSchema;
import org.apache.ws.jaxme.xs.XSType;
import org.apache.ws.jaxme.xs.parser.XSContext;
import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.apache.ws.jaxme.xs.types.*;
import org.apache.ws.jaxme.xs.xml.XsESchema;
import org.apache.ws.jaxme.xs.xml.XsQName;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/** <p>Implementation of an XML Schema, as defined by the
 * {@link XSSchema} interface.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XSSchemaImpl implements XSSchema {
  private static final XSType[] BUILTIN_TYPES = new XSType[]{
    XSAnySimpleType.getInstance(),
    XSAnyURI.getInstance(),
    XSBase64Binary.getInstance(),
    XSBoolean.getInstance(),
    XSByte.getInstance(),
    XSDate.getInstance(),
    XSDateTime.getInstance(),
    XSDecimal.getInstance(),
    XSDouble.getInstance(),
    XSDuration.getInstance(),
    XSEntities.getInstance(),
    XSEntity.getInstance(),
    XSFloat.getInstance(),
    XSGDay.getInstance(),
    XSGMonth.getInstance(),
    XSGMonthDay.getInstance(),
    XSGYear.getInstance(),
    XSGYearMonth.getInstance(),
    XSHexBinary.getInstance(),
    XSID.getInstance(),
    XSIDREF.getInstance(),
    XSIDREFs.getInstance(),
    XSInt.getInstance(),
    XSInteger.getInstance(),
    XSLanguage.getInstance(),
    XSLong.getInstance(),
    XSName.getInstance(),
    XSNCName.getInstance(),
    XSNegativeInteger.getInstance(),
    XSNMToken.getInstance(),
    XSNMTokens.getInstance(),
    XSNonNegativeInteger.getInstance(),
    XSNonPositiveInteger.getInstance(),
    XSNormalizedString.getInstance(),
    org.apache.ws.jaxme.xs.types.XSNotation.getInstance(),
    XSPositiveInteger.getInstance(),
    XSQName.getInstance(),
    XSShort.getInstance(),
    XSString.getInstance(),
    XSTime.getInstance(),
    XSToken.getInstance(),
    XSUnsignedByte.getInstance(),
    XSUnsignedInt.getInstance(),
    XSUnsignedLong.getInstance(),
    XSUnsignedShort.getInstance(),
    XSAnyType.getInstance()
  };

  private final XSContext context;
  private final XsESchema syntaxSchema;
  private final List childs = new ArrayList();
  private final Map types = new HashMap();
  private final Map builtinTypes = new HashMap();
  private final Map groups = new HashMap();
  private final Map attributeGroups = new HashMap();
  private final Map attributes = new HashMap();
  private final Map elements = new HashMap();
  private final Attributes openAttrs;
  private boolean isValidated;

  public XSSchemaImpl(XSContext pContext, XsESchema pSchema) {
    context = pContext;
    syntaxSchema = pSchema;
    for (int i = 0;  i < BUILTIN_TYPES.length;  i++) {
      builtinTypes.put(BUILTIN_TYPES[i].getName(), BUILTIN_TYPES[i]);
    }
    openAttrs = pSchema.getOpenAttributes();
  }

  public XSContext getContext() { return context; }
  public boolean isTopLevelObject() { return true; }
  public XSObject getParentObject() { return null; }
  public XSObjectFactory getXSObjectFactory() { return context.getXSObjectFactory(); }
  public Locator getLocator() { return syntaxSchema.getLocator(); }
  public XSSchema getXSSchema() { return this; }
  protected XsESchema getXsESchema() { return syntaxSchema; }

  /** <p>Adds a new child to the array returned by {@link #getChilds()}.</p>
   */
  protected void addChild(Object pChild) {
    childs.add(pChild);
  }

  /** <p>Replaces the existing child <code>pOldChild</code> with
   * the replacement object <code>pNewChild</code>. This method
   * is used from within the various <code>redefine()</code> methods.</p>
   */
  protected void replace(Object pOldChild, Object pNewChild) {
    for (ListIterator iter = childs.listIterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o.equals(pOldChild)) {
        iter.set(pNewChild);
        return;
      }
    }
    throw new NoSuchElementException();
  }

  public XSAnnotation[] getAnnotations() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSAnnotation) {
        result.add(o);
      }
    }
    return (XSAnnotation[]) result.toArray(new XSAnnotation[result.size()]);
  }

  public XSType[] getTypes() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSType) {
        result.add(o);
      }
    }
    return (XSType[]) result.toArray(new XSType[result.size()]);
  }

  public XSType[] getBuiltinTypes() {
    return BUILTIN_TYPES;
  }

  public XSType getType(XsQName pName) {
    XSType result = (XSType) types.get(pName);
    if (result == null) {
      result = (XSType) builtinTypes.get(pName);
    }
    return result;
  }

  public XSGroup[] getGroups() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSGroup) {
        result.add(o);
      }
    }
    return (XSGroup[]) result.toArray(new XSGroup[result.size()]);
  }

  public XSGroup getGroup(XsQName pName) {
    return (XSGroup) groups.get(pName);
  }

  public XSAttributeGroup[] getAttributeGroups() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSAttributeGroup) {
        result.add(o);
      }
    }
    return (XSAttributeGroup[]) result.toArray(new XSAttributeGroup[result.size()]);
  }

  public XSAttributeGroup getAttributeGroup(XsQName pName) {
    return (XSAttributeGroup) attributeGroups.get(pName);
  }

  public XSElement[] getElements() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSElement) {
        result.add(o);
      }
    }
    return (XSElement[]) result.toArray(new XSElement[result.size()]);
  }

  public XSElement getElement(XsQName pName) {
    return (XSElement) elements.get(pName);
  }

  public XSAttribute[] getAttributes() {
    List result = new ArrayList();
    for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
      Object o = iter.next();
      if (o instanceof XSAttribute) {
        result.add(o);
      }
    }
    return (XSAttribute[]) result.toArray(new XSAttribute[result.size()]);
  }

  public XSAttribute getAttribute(XsQName pQName) {
    return (XSAttribute) attributes.get(pQName);
  }

  public void add(XSAnnotation pAnnotation) {
    addChild(pAnnotation);
  }

  public void add(XSType pType) throws SAXException {
    XsQName name = pType.getName();
    if (name == null) {
      throw new LocSAXException("A global type must have a 'name' attribute.", pType.getLocator());
    }
    if (types.containsKey(name)) {
      throw new LocSAXException("A global type " + name + " is already defined.", pType.getLocator());
    }
    types.put(name, pType);
    pType.setGlobal(true);
    addChild(pType);
  }

  public void redefine(XSType pType) throws SAXException {
    XsQName name = pType.getName();
    if (name == null) {
      throw new LocSAXException("A global type must have a 'name' attribute.", pType.getLocator());
    }
    Object oldType = types.get(name);
    if (oldType == null) {
      throw new LocSAXException("The global type " + name + " cannot be redefined, as it doesn't yet exist.",
                                   pType.getLocator());
    }
    types.put(name, pType);
    pType.setGlobal(true);
    replace(oldType, pType);
  }

  public void add(XSGroup pGroup) throws SAXException {
    XsQName name = pGroup.getName();
    if (name == null) {
      throw new LocSAXException("A global group must have a 'name' attribute.", pGroup.getLocator());
    }
    if (groups.containsKey(name)) {
      throw new LocSAXException("A global group " + name + " is already defined.", pGroup.getLocator());
    }
    groups.put(name, pGroup);
    addChild(pGroup);
    pGroup.setGlobal(true);
  }

  public void redefine(XSGroup pGroup) throws SAXException {
    XsQName name = pGroup.getName();
    if (name == null) {
      throw new LocSAXException("A global group must have a 'name' attribute.", pGroup.getLocator());
    }
    Object oldGroup = groups.get(name);
    if (oldGroup == null) {
      throw new LocSAXException("The global group " + name + " cannot be redefined, as it doesn't yet exist.",
                                   pGroup.getLocator());
    }
    groups.put(name, pGroup);
    replace(oldGroup, pGroup);
  }

  public void add(XSAttributeGroup pGroup) throws SAXException {
    XsQName name = pGroup.getName();
    if (name == null) {
      throw new LocSAXException("A global attribute group must have a 'name' attribute.", pGroup.getLocator());
    }
    if (attributeGroups.containsKey(name)) {
      throw new LocSAXException("A global attribute group " + name + " is already defined.", pGroup.getLocator());
    }
    attributeGroups.put(name, pGroup);
    addChild(pGroup);
  }

  public void redefine(XSAttributeGroup pGroup) throws SAXException {
    XsQName name = pGroup.getName();
    if (name == null) {
      throw new LocSAXException("A global attribute group must have a 'name' attribute.", pGroup.getLocator());
    }
    Object oldGroup = attributeGroups.get(name);
    if (!attributeGroups.containsKey(name)) {
      throw new LocSAXException("The global attribute group " + name + " cannot be redefined, as it doesn't yet exist.",
                                   pGroup.getLocator());
    }
    attributeGroups.put(name, pGroup);
    replace(oldGroup, pGroup);
  }

  public void add(XSAttribute pAttribute) throws SAXException {
    XsQName name = pAttribute.getName();
    if (name == null) {
      throw new LocSAXException("A global attribute must have a 'name' attribute.", pAttribute.getLocator());
    }
    if (attributes.containsKey(name)) {
      throw new LocSAXException("A global attribute " + name + " is already defined.", pAttribute.getLocator());
    }
    attributes.put(name, pAttribute);
    addChild(pAttribute);
  }

  public void add(XSElement pElement) throws SAXException {
    XsQName name = pElement.getName();
    if (name == null) {
      throw new LocSAXException("A global attribute group must have a 'name' attribute.", pElement.getLocator());
    }
    if (elements.containsKey(name)) {
      throw new LocSAXException("A global group " + name + " is already defined.", pElement.getLocator());
    }
    elements.put(name, pElement);
    addChild(pElement);
  }

  public void add(XSNotation pNotation) {
    addChild(pNotation);
  }

  public Object[] getChilds() {
    return childs.toArray();
  }

  protected void validate(Object pChild) throws SAXException {
    if (pChild instanceof XSObject) {
      ((XSObject) pChild).validate();
    } else {
      throw new IllegalStateException("Unable to validate the child " + pChild +
                                       ", perhaps you should overwrite the method " +
                                       getClass().getName() + ".validate(Object).");
    }
  }

  protected boolean isValidated() {
    return isValidated;
  }

  public void validate() throws SAXException {
    if (isValidated()) {
      return;
    } else {
      isValidated = true;
    }

    Object[] myChilds = getChilds();
    for (int i = 0;  i < myChilds.length;  i++) {
      validate(myChilds[i]);
    }
  }

  public Attributes getOpenAttributes() {
    return openAttrs;
  }
}
