/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.parser.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.UndeclaredThrowableException;

import org.apache.ws.jaxme.xs.XSParser;
import org.apache.ws.jaxme.xs.parser.*;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;


/** <p>Default implementation of a {@link org.apache.ws.jaxme.xs.parser.ChildSetter}.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class ChildSetterImpl implements ChildSetter {
  private static final Class[] GETCHILDHANDLER_CLASSES = new Class[]{String.class, String.class, String.class};
  private static final Class[] ZERO_CLASSES = new Class[]{};
  private static final Object[] ZERO_OBJECTS = new Object[]{};

  private XSContext getData() {
    return XSParser.getRunningInstance().getContext();
  }

  /** <p>This method invokes the beans <code>pBean</code> method <code>pMethod</code> with
   * the argument array pArgs, returning a child handler for the element <code>pQName</code>.</p>
   */
  protected Object invokeMethod(Object pBean, Method pMethod, String pName, Object[] pArgs) throws SAXException {
    try {
      return pMethod.invoke(pBean, pArgs);
    } catch (InvocationTargetException e) {
      Throwable t = e.getTargetException();
      if (t instanceof RuntimeException) {
        throw (RuntimeException) t;
      } else if (t instanceof SAXException) {
        throw (SAXException) t;
      } else {
        throw new UndeclaredThrowableException(t);
      }
    } catch (IllegalAccessException e) {
      throw new IllegalStateException("Failed to invoke method " + pMethod.getName() +
                                       " of class " + pMethod.getDeclaringClass() +
                                       " with argument " + pBean + ": IllegalAccessException, " +
                                       e.getMessage()); 
    }
  }


  public ContentHandler getChildHandler(String pQName,
                                         String pNamespaceURI, String pLocalName) throws SAXException {
    final XsSAXParser xsSAXParser = (XsSAXParser) getData().getCurrentContentHandler();
    if (xsSAXParser == null) {
      throw new NullPointerException("No XsSAXParser registered.");
    }
    final Object bean = xsSAXParser.getBean();
    final Class beanClass = bean.getClass();
    try {
      Method m = beanClass.getMethod("getChildHandler", GETCHILDHANDLER_CLASSES);
      if (Modifier.isPublic(m.getModifiers())  &&
          ContentHandler.class.isAssignableFrom(m.getReturnType())) {
        Object result = (ContentHandler) invokeMethod(bean, m, pQName, new Object[]{pQName, pNamespaceURI, pLocalName});
        if (result != null) {
          return (ContentHandler) result;
        }
      }
    } catch (NoSuchMethodException e) {
    }

    final ContentHandler result = getChildHandler(xsSAXParser, pQName, pLocalName);
    if (result == null) {
      throw new LocSAXException("Unknown child element '" + pQName + "' for bean " + bean.getClass().getName(),
                                 getData().getLocator());
    }
    return result;
  }

  /** <p>Creates a new instance of {@link XsSAXParser}, inheriting most properties from
   * its parent parser.</p>
   */
  protected ContentHandler newXsSAXParser(XsSAXParser pParent, Object pBean) {
    return getData().getXsObjectFactory().newXsSAXParser(pBean);
  }

  protected ContentHandler getChildHandler(ContentHandler pParent, String pQName, String pLocalName)
      throws SAXException {
    final XsSAXParser xsSAXParser = (XsSAXParser) pParent;
    final Object bean = xsSAXParser.getBean();
    final Class beanClass = bean.getClass();
    final String s = Character.toUpperCase(pLocalName.charAt(0)) + pLocalName.substring(1);
    try {
      Method m = beanClass.getMethod("create" + s, ZERO_CLASSES);
      if (Modifier.isPublic(m.getModifiers())  &&
         !void.class.equals(m.getReturnType())) {
        Object result = invokeMethod(bean, m, pQName, ZERO_OBJECTS);
        return newXsSAXParser(xsSAXParser, result);
      }
    } catch (NoSuchMethodException e) {
    } catch (SAXException e) {
      throw e;
    }

    return null;
  }
}
