/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml;

import java.util.StringTokenizer;

/** <p>Implementation of <code>xs:derivationset</code>.
 * Follows this specification:
 * <pre>
 *  &lt;xs:simpleType name="derivationSet"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation&gt;
 *        A utility type, not for public use
 *      &lt;/xs:documentation&gt;
 *      &lt;xs:documentation&gt;
 *        #all or (possibly empty) subset of {extension, restriction}
 *      &lt;/xs:documentation&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:union&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:restriction base="xs:token"&gt;
 *          &lt;xs:enumeration value="#all"/&gt;
 *        &lt;/xs:restriction&gt;
 *      &lt;/xs:simpleType&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:list itemType="xs:reducedDerivationControl"/&gt;
 *      &lt;/xs:simpleType&gt;
 *    &lt;/xs:union&gt;
 *  &lt;/xs:simpleType&gt;
 * </pre>
 * </p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsDerivationSet {
  boolean extensionAllowed, restrictionAllowed;

  /** <p>Returns whether extension is allowed.</p>
   */
  public boolean isExtensionAllowed() {
    return extensionAllowed;
  }

  /** <p>Sets whether extension is allowed.</p>
   */
  public void setExtensionAllowed(boolean pExtensionAllowed) {
    extensionAllowed = pExtensionAllowed;
  }

  /** <p>Returns whether restriction is allowed.</p>
   */
  public boolean isRestrictionAllowed() {
    return restrictionAllowed;
  }

  /** <p>Sets whether restriction is allowed.</p>
   */
  public void setRestrictionAllowed(boolean pRestrictionAllowed) {
    restrictionAllowed = pRestrictionAllowed;
  }

  /** <p>Returns a <code>DerivationSet</code> matching the given
   * value.</p>
   */
  public static XsDerivationSet valueOf(String pValue) {
    return new XsDerivationSet(pValue);
  }

  /** <p>Creates a new DerivationSet with the given value.</p>
   */
  public XsDerivationSet(String pValue) {
    if ("#all".equals(pValue)) {
      setExtensionAllowed(true);
      setRestrictionAllowed(true);
    } else {
      for (StringTokenizer st = new StringTokenizer(pValue, " ");  st.hasMoreTokens();  ) {
        String s = st.nextToken();
        if ("extension".equals(s)) {
          setExtensionAllowed(true);
        } else if ("restriction".equals(s)) {
          setRestrictionAllowed(true);
        } else {
          throw new IllegalArgumentException("Invalid derivation set value: " + pValue + "; the token " + s + " did not resolve to either of 'extension' or 'restriction'");
        }
      }
    }
  }

  public String toString() {
    if (isExtensionAllowed()) {
      if (isRestrictionAllowed()) {
        return "extension restriction";
      } else {
        return "extension";
      }
    } else {
      if (isRestrictionAllowed()) {
        return "restriction";
      } else {
        return "";
      }
    }
  }

  public boolean equals(Object o) {
    if (o == null  ||  !(XsDerivationSet.class.equals(o.getClass()))) {
      return false;
    }
    XsDerivationSet ds = (XsDerivationSet) o;
    return ds.extensionAllowed == extensionAllowed  &&
            ds.restrictionAllowed == restrictionAllowed;
  }

  public int hashCode() {
    int result = 0;
    if (extensionAllowed) { result += 1; }
    if (restrictionAllowed) { result += 2; }
    return result;
  }
}
