/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/** <p>Implementation of the <code>xs:namespaceList</code> type,
 * specified like this:
 * <pre>
 *  &lt;xs:simpleType name="namespaceList"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation&gt;
 *        A utility type, not for public use
 *      &lt;/xs:documentation&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:union&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:restriction base="xs:token"&gt;
 *          &lt;xs:enumeration value="##any"/&gt;
 *          &lt;xs:enumeration value="##other"/&gt;
 *        &lt;/xs:restriction&gt;
 *      &lt;/xs:simpleType&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:list&gt;
 *          &lt;xs:simpleType&gt;
 *            &lt;xs:union memberTypes="xs:anyURI"&gt;
 *              &lt;xs:simpleType&gt;
 *                &lt;xs:restriction base="xs:token"&gt;
 *                  &lt;xs:enumeration value="##targetNamespace"/&gt;
 *                   &lt;xs:enumeration value="##local"/&gt;
 *                &lt;/xs:restriction&gt;
 *              &lt;/xs:simpleType&gt;
 *            &lt;/xs:union&gt;
 *          &lt;/xs:simpleType&gt;
 *        &lt;/xs:list&gt;
 *      &lt;/xs:simpleType&gt;
 *    &lt;/xs:union&gt;
 *  &lt;/xs:simpleType&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsNamespaceList {
  public static final XsNamespaceList ANY = new XsNamespaceList(){
    public boolean isAny(){ return true; }
  };
  public static final XsNamespaceList OTHER = new XsNamespaceList(){
    public boolean isOther(){ return true; }
  };

  private boolean isTargetNamespaceSet, isLocalSet;
  private List uris;

  XsNamespaceList(){
  }

  public static XsNamespaceList valueOf(String pValue) {
    if ("##any".equals(pValue)) {
      return ANY;
    } else if ("##other".equals(pValue)) {
      return OTHER;
    } else {
      XsNamespaceList result = new XsNamespaceList();
      result.uris = new ArrayList();
      for (StringTokenizer st = new StringTokenizer(pValue);  st.hasMoreTokens();  ) {
        String s = st.nextToken();
        if ("##targetNamespace".equals(s)) {
          result.isTargetNamespaceSet = true;
        } else if ("##local".equals(s)) {
          result.isLocalSet = true;
        } else {
          result.uris.add(new XsAnyURI(s));
        }
      }
      return result;
    }
  }

  /** <p>Returns whether the namespace list matches <code>##any</code>.
   * This is the case if, and only if, {@link #isOther()} returns
   * false, and {@link #getUris()} returns null.</p>
   */
  public boolean isAny() { return false; }

  /** <p>Returns whether the namespace list matches <code>##other</code>.
   * This is the case if, and only if, {@link #isAny()} returns
   * false, and {@link #getUris()} returns null.</p>
   */
  public boolean isOther() { return false; }

  /** <p>Returns the array of URI's specified in the namespace list.
   * Returns non-null if, and only if, {@link #isAny()} and
   * {@link #isOther()} return false.</p>
   * <p>If this method returns non-null, you may also use the methods
   * {@link #isTargetNamespaceSet()} and {@link #isLocalSet()}.</p>
   */
  public XsAnyURI[] getUris() {
    if (uris == null) {
      return null;
    }
    return (XsAnyURI[]) uris.toArray(new XsAnyURI[uris.size()]);
  }

  /** <p>Returns whether the URI list did contain the value
   * <code>##targetNamespace</code>.</p>
   */
  public boolean isTargetNamespaceSet() {
    return isTargetNamespaceSet;
  }

  /** <p>Returns whether the URI list did contain the value
   * <code>##targetNamespace</code>.</p>
   */
  public boolean isLocalSet() {
    return isLocalSet;
  }
}
