/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml;

import java.util.StringTokenizer;

/** <p>Implementation of <code>xs:derivationset</code>.
 * Follows this specification:
 * <pre>
 *  &lt;xs:simpleType name="simpleDerivationSet"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation&gt;
 *        #all or (possibly empty) subset of {restriction, union, list}
 *      &lt;/xs:documentation&gt;
 *      &lt;xs:documentation&gt;
 *        A utility type, not for public use
 *      &lt;/xs:documentation&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:union&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:restriction base="xs:token"&gt;
 *          &lt;xs:enumeration value="#all"/&gt;
 *        &lt;/xs:restriction&gt;
 *      &lt;/xs:simpleType&gt;
 *      &lt;xs:simpleType&gt;
 *        &lt;xs:restriction base="xs:derivationControl"&gt;
 *          &lt;xs:enumeration value="list"/&gt;
 *          &lt;xs:enumeration value="union"/&gt;
 *          &lt;xs:enumeration value="restriction"/&gt;
 *        &lt;/xs:restriction&gt;
 *      &lt;/xs:simpleType&gt;
 *    &lt;/xs:union&gt;
 *  &lt;/xs:simpleType&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsSimpleDerivationSet {
  final boolean listAllowed, unionAllowed, restrictionAllowed;

  /** <p>Returns whether derivation of lists is allowed.</p>
   */
  public boolean isListAllowed() {
    return listAllowed;
  }

  /** <p>Returns whether derivation of unions is allowed.</p>
   */
  public boolean isUnionAllowed() {
    return unionAllowed;
  }

  /** <p>Returns whether derivation of restrictions is allowed.</p>
   */
  public boolean isRestrictionAllowed() {
    return restrictionAllowed;
  }

  /** <p>Returns a <code>DerivationSet</code> matching the given
   * value.</p>
   */
  public static XsSimpleDerivationSet valueOf(String pValue) {
    return new XsSimpleDerivationSet(pValue);
  }

  /** <p>Creates a new DerivationSet with the given value.</p>
   */
  public XsSimpleDerivationSet(String pValue) {
    if ("#all".equals(pValue)) {
      listAllowed = true;
      unionAllowed = true;
      restrictionAllowed = true;
    } else {
      boolean acceptList = false;
      boolean acceptUnion = false;
      boolean acceptRestriction = false;
      for (StringTokenizer st = new StringTokenizer(pValue, " ");  st.hasMoreTokens();  ) {
        String s = st.nextToken();
        if ("list".equals(s)) {
          acceptList = true;
        } else if ("union".equals(s)) {
          acceptUnion = true;
        } else if ("restriction".equals(s)) {
          acceptRestriction = true;
        } else {
          throw new IllegalArgumentException("Invalid derivation set value: " + pValue + "; the token " + s + " did not resolve to either of 'extension' or 'restriction'");
        }
      }
      listAllowed = acceptList;
      unionAllowed = acceptUnion;
      restrictionAllowed = acceptRestriction;
    }
  }

  public String toString() {
    final StringBuffer sb = new StringBuffer();
    if (listAllowed) {
      sb.append("list");
    }
    if (unionAllowed) {
      if (sb.length() > 0) {
        sb.append(" ");
      }
      sb.append("union");
    }
    if (restrictionAllowed) {
      if (sb.length() > 0) {
        sb.append(" ");
      }
      sb.append("restriction");
    }
    return sb.toString();
  }

  public boolean equals(Object o) {
    if (o == null  ||  !(XsSimpleDerivationSet.class.equals(o.getClass()))) {
      return false;
    }
    XsSimpleDerivationSet ds = (XsSimpleDerivationSet) o;
    return ds.listAllowed == listAllowed  &&
            ds.unionAllowed == unionAllowed  &&
            ds.restrictionAllowed == restrictionAllowed;
  }

  public int hashCode() {
    int result = 0;
    if (listAllowed) { result += 1; }
    if (unionAllowed) { result += 2; }
    if (restrictionAllowed) { result += 4; }
    return result;
  }
}
