/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.ws.jaxme.xs.XSParser;
import org.apache.ws.jaxme.xs.parser.DOMBuilder;
import org.apache.ws.jaxme.xs.xml.*;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;


/** <p>Implementation of <code>xs:annotation</code>, as specified
 * by the following:
 * <pre>
 *  &lt;xs:element name="annotation" id="annotation"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation source="http://www.w3.org/TR/xmlschema-1/#element-annotation"/&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:complexType&gt;
 *      &lt;xs:complexContent&gt;
 *        &lt;xs:extension base="xs:openAttrs"&gt;
 *          &lt;xs:choice minOccurs="0" maxOccurs="unbounded"&gt;
 *            &lt;xs:element ref="xs:appinfo"/&gt;
 *            &lt;xs:element ref="xs:documentation"/&gt;
 *          &lt;/xs:choice&gt;
 *          &lt;xs:attribute name="id" type="xs:ID"/&gt;
 *        &lt;/xs:extension&gt;
 *      &lt;/xs:complexContent&gt;
 *    &lt;/xs:complexType&gt;
 *  &lt;/xs:element&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsEAnnotationImpl extends XsTOpenAttrsImpl implements XsEAnnotation {
  private XsID id;
  private List childs;

  protected XsEAnnotationImpl(XsObject pParent) {
    super(pParent);
  }

  public void setId(XsID pId) {
    id = pId;
  }

  public XsID getId() {
    return id;
  }

  protected void addChild(Object pChild) {
    if (childs == null) {
      childs = new ArrayList();
    }
    childs.add(pChild);
  }

  public XsEAppinfo createAppinfo() {
    XsEAppinfo appinfo = getObjectFactory().newXsEAppinfo(this);
    addChild(appinfo);
    return appinfo;
  }

  public XsEDocumentation createDocumentation() {
    XsEDocumentation documentation = getObjectFactory().newXsEDocumentation(this);
    addChild(documentation);
    return documentation;
  }

  public XsEAppinfo[] getAppinfos() {
    if (childs == null) {
      return new XsEAppinfo[0];
    } else {
      List result = new ArrayList();
      for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
        Object o = iter.next();
        if (o instanceof XsEAppinfo) {
          result.add(o);
        }
      }
      return (XsEAppinfo[]) result.toArray(new XsEAppinfo[result.size()]);
    }
  }

  public XsEDocumentation[] getDocumentations() {
    if (childs == null) {
      return new XsEDocumentation[0];
    } else {
      List result = new ArrayList();
      for (Iterator iter = childs.iterator();  iter.hasNext();  ) {
        Object o = iter.next();
        if (o instanceof XsEDocumentation) {
          result.add(o);
        }
      }
      return (XsEDocumentation[]) result.toArray(new XsEDocumentation[result.size()]);
    }
  }

  public Object[] getChilds() {
    if (childs == null) {
      return new Object[0];
    } else {
      return childs.toArray();
    }
  }

  public ContentHandler getChildHandler(String pQName,
                                         String pNamespaceURI, String pLocalName) throws SAXException {
    if (XSParser.XML_SCHEMA_URI.equals(pNamespaceURI)) {
      return null;  // Let the parser handle this element
    }

    try {
      DOMBuilder db = new DOMBuilder();
      addChild(db.getDocument());
      return db;
    } catch (ParserConfigurationException e) {
      throw new SAXException(e);
    }
  }
}
