/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml.impl;

import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.apache.ws.jaxme.xs.xml.*;
import org.xml.sax.SAXException;


/** <p>Implementation of the group <code>xs:complexTypeModel</code>,
 * as specified by the following:
 * <pre>
 *  &lt;xs:group name="complexTypeModel"&gt;
 *    &lt;xs:choice&gt;
 *      &lt;xs:element ref="xs:simpleContent"/&gt;
 *      &lt;xs:element ref="xs:complexContent"/&gt;
 *      &lt;xs:sequence&gt;
 *        &lt;xs:annotation&gt;
 *          &lt;xs:documentation&gt;
 *            This branch is short for &amp;lt;complexContent&amp;gt;
 *            &amp;lt;restriction base="xs:anyType"&amp;gt;
 *            ...
 *            &amp;lt;/restriction&amp;gt;
 *            &amp;lt;/complexContent&amp;gt;
 *          &lt;/xs:documentation&gt;
 *        &lt;/xs:annotation&gt;
 *        &lt;xs:group ref="xs:typeDefParticle" minOccurs="0"/&gt;
 *        &lt;xs:group ref="xs:attrDecls"/&gt;
 *      &lt;/xs:sequence&gt;
 *    &lt;/xs:choice&gt;
 *  &lt;/xs:group&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsGComplexTypeModelImpl implements XsGComplexTypeModel {
  private final XsObject owner;
  private XsESimpleContent simpleContent;
  private XsEComplexContent complexContent;
  private boolean otherContent;
  private final XsGTypeDefParticle typeDefParticle;
  private final XsGAttrDecls attrDecls;

  protected XsGComplexTypeModelImpl(XsObject pOwner) {
    owner = pOwner;
    XsObjectFactory factory = owner.getObjectFactory();
    typeDefParticle = factory.newXsGTypeDefParticle(owner);
    attrDecls = factory.newXsGAttrDecls(owner);
  }

  public XsESimpleContent createSimpleContent() {
    if (simpleContent != null) {
      throw new IllegalStateException("Multiple 'simpleContent' child elements are forbidden.");
    }
    if (complexContent != null) {
      throw new IllegalStateException("The 'simpleContent' and 'complexContent' child elements are mutually exclusive.");
    }
    if (otherContent) {
      throw new IllegalStateException("The 'simpleContent' child element and the child elements 'all', 'choice', 'sequence', 'group', 'attribute', and 'attributeGroup' are mutually exclusive.");
    }
    return simpleContent = owner.getObjectFactory().newXsESimpleContent(owner);
  }

  public XsESimpleContent getSimpleContent() {
    return simpleContent;
  }

  public XsEComplexContent createComplexContent() {
    if (complexContent != null) {
      throw new IllegalStateException("Multiple 'complexContent' child elements are forbidden.");
    }
    if (simpleContent != null) {
      throw new IllegalStateException("The 'complexContent' and 'simpleContent' child elements are mutually exclusive.");
    }
    if (otherContent) {
      throw new IllegalStateException("The 'complexContent' child element and the child elements 'all', 'choice', 'sequence', 'group', 'attribute', and 'attributeGroup' are mutually exclusive.");
    }
    return complexContent = owner.getObjectFactory().newXsEComplexContent(owner);
  }

  public XsEComplexContent getComplexContent() {
    return complexContent;
  }

  protected void validateOtherContent() {
    if (simpleContent != null) {
      throw new IllegalStateException("The child elements 'all', 'choice', 'sequence', 'group', 'attribute', 'attributeGroup', and the child element 'simpleContent' are mutually exclusive.");
    }
    if (complexContent != null) {
      throw new IllegalStateException("The child elements 'all', 'choice', 'sequence', 'group', 'attribute', 'attributeGroup', and the child element 'complexContent' are mutually exclusive.");
    }
  }

  public XsTAttribute createAttribute() {
    validateOtherContent();
    return attrDecls.createAttribute();
  }

  public XsTAttribute[] getAttributes() {
    return attrDecls.getAttributes();
  }

  public XsTAttributeGroupRef createAttributeGroup() {
    validateOtherContent();
    return attrDecls.createAttributeGroup();
  }

  public XsTAttributeGroupRef[] getAttributeGroups() {
    return attrDecls.getAttributeGroups();
  }

  public XsTWildcard createAnyAttribute() {
    validateOtherContent();
    return attrDecls.createAnyAttribute();
  }

  public XsTWildcard getAnyAttribute() {
    return attrDecls.getAnyAttribute();
  }

  public Object[] getAllAttributes() {
    return attrDecls.getAllAttributes();
  }

  public void validate() throws SAXException {
    if (!otherContent  &&  (simpleContent == null  &&  complexContent == null)) {
      throw new LocSAXException("You must specify either of the 'simpleContent', 'complexContent', 'all', 'choice', 'sequence', or 'group' child elements.",
                                 owner.getLocator());
    }
  }

  public XsTGroupRef createGroup() {
    return typeDefParticle.createGroup();
  }

  public XsTAll createAll() {
    return typeDefParticle.createAll();
  }

  public XsESequence createSequence() {
    return typeDefParticle.createSequence();
  }

  public XsEChoice createChoice() {
    return typeDefParticle.createChoice();
  }

  public XsTTypeDefParticle getTypeDefParticle() {
    return typeDefParticle.getTypeDefParticle();
  }
}
