/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.ws.jaxme.xs.xml.*;


/** <p>Interface of <code>xs:simpleRestrictionModel</code>, following
 * this specification:
 * <pre>
 *  &lt;xs:group name="simpleRestrictionModel"&gt;
 *    &lt;xs:sequence&gt;
 *      &lt;xs:element name="simpleType" type="xs:localSimpleType" minOccurs="0"/&gt;
 *      &lt;xs:group ref="xs:facets" minOccurs="0" maxOccurs="unbounded"/&gt;
 *    &lt;/xs:sequence&gt;
 *  &lt;/xs:group&gt;
 *
 *  &lt;xs:group name="facets"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation&gt;
 *        We should use a substitution group for facets, but
 *        that's ruled out because it would allow users to
 *        add their own, which we're not ready for yet.
 *      &lt;/xs:documentation&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:choice&gt;
 *      &lt;xs:element ref="xs:minExclusive"/&gt;
 *      &lt;xs:element ref="xs:minInclusive"/&gt;
 *      &lt;xs:element ref="xs:maxExclusive"/&gt;
 *      &lt;xs:element ref="xs:maxInclusive"/&gt;
 *      &lt;xs:element ref="xs:totalDigits"/&gt;
 *      &lt;xs:element ref="xs:fractionDigits"/&gt;
 *      &lt;xs:element ref="xs:length"/&gt;
 *      &lt;xs:element ref="xs:minLength"/&gt;
 *      &lt;xs:element ref="xs:maxLength"/&gt;
 *      &lt;xs:element ref="xs:enumeration"/&gt;
 *      &lt;xs:element ref="xs:whiteSpace"/&gt;
 *      &lt;xs:element ref="xs:pattern"/&gt;
 *    &lt;/xs:choice&gt;
 *  &lt;/xs:group&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsGSimpleRestrictionModelImpl implements XsGSimpleRestrictionModel {
  private final XsObject owner;
  private XsTLocalSimpleType simpleType;
  private List facets;

  protected XsGSimpleRestrictionModelImpl(XsObject pOwner) {
    owner = pOwner;
  }

  public XsTLocalSimpleType createSimpleType() {
    if (simpleType != null) {
      throw new IllegalStateException("Multiple 'simpleType' childs are forbidden.");
    }
    if (facets != null  &&  facets.size() > 0) {
      throw new IllegalStateException("The 'simpleType' child element must precede the '" +
                                       ((XsTFacetBase) facets.get(0)).getFacetName() + "' child element.");
    }
    return simpleType = owner.getObjectFactory().newXsTLocalSimpleType(owner);
  }

  public XsTLocalSimpleType getSimpleType() {
    return simpleType;
  }

  protected XsTFacetBase getFacetByName(String pName) {
    if (facets != null) {
      for (int i = 0;  i < facets.size();  i++) {
        XsTFacetBase facet = (XsTFacetBase) facets.get(i);
        if (pName.equals(facet.getFacetName())) {
          return facet;
        }
      }
    }
    return null;
  }

  protected void addFacet(XsTFacetBase pFacet) {
    if (facets == null) {
      facets = new ArrayList();
    }
    facets.add(pFacet);
  }

  protected void addUniqueFacet(XsTFacetBase pFacet) {
    if (getFacetByName(pFacet.getFacetName()) != null) {
      throw new IllegalStateException("Multiple '" + pFacet.getFacetName() + "' childs are forbidden.");
    }
    addFacet(pFacet);
  }

  public XsEMinExclusive createMinExclusive() {
    XsEMinExclusive minExclusive = owner.getObjectFactory().newXsEMinExclusive(owner);
    addUniqueFacet(minExclusive);
    return minExclusive;
  }

  public XsEMinExclusive getMinExclusive() {
    return (XsEMinExclusive) getFacetByName("minExclusive");
  }

  public XsEMinInclusive createMinInclusive() {
    XsEMinInclusive minInclusive = owner.getObjectFactory().newXsEMinInclusive(owner);
    addUniqueFacet(minInclusive);
    return minInclusive;
  }

  public XsEMinInclusive getMinInclusive() {
    return (XsEMinInclusive) getFacetByName("minInclusive");
  }

  public XsEMaxExclusive createMaxExclusive() {
    XsEMaxExclusive maxExclusive = owner.getObjectFactory().newXsEMaxExclusive(owner);
    addUniqueFacet(maxExclusive);
    return maxExclusive;
  }

  public XsEMaxExclusive getMaxExclusive() {
    return (XsEMaxExclusive) getFacetByName("maxExclusive");
  }

  public XsEMaxInclusive createMaxInclusive() {
    XsEMaxInclusive maxInclusive = owner.getObjectFactory().newXsEMaxInclusive(owner);
    addUniqueFacet(maxInclusive);
    return maxInclusive;
  }

  public XsEMaxInclusive getMaxInclusive() {
    return (XsEMaxInclusive) getFacetByName("maxInclusive");
  }

  public XsETotalDigits createTotalDigits() {
    XsETotalDigits totalDigits = owner.getObjectFactory().newXsETotalDigits(owner);
    addUniqueFacet(totalDigits);
    return totalDigits;
  }

  public XsETotalDigits getTotalDigits() {
    return (XsETotalDigits) getFacetByName("totalDigits");
  }

  public XsEFractionDigits createFractionDigits() {
    XsEFractionDigits fractionDigits = owner.getObjectFactory().newXsEFractionDigits(owner);
    addUniqueFacet(fractionDigits);
    return fractionDigits;
  }

  public XsEFractionDigits getFractionDigits() {
    return (XsEFractionDigits) getFacetByName("fractionDigits");
  }

  public XsELength createLength() {
    XsELength length = owner.getObjectFactory().newXsELength(owner);
    addUniqueFacet(length);
    return length;
  }

  public XsELength getLength() {
    return (XsELength) getFacetByName("length");
  }

  public XsEMinLength createMinLength() {
    XsEMinLength minLength = owner.getObjectFactory().newXsEMinLength(owner);
    addUniqueFacet(minLength);
    return minLength;
  }

  public XsEMinLength getMinLength() {
    return (XsEMinLength) getFacetByName("minLength");
  }

  public XsEMaxLength createMaxLength() {
    XsEMaxLength maxLength = owner.getObjectFactory().newXsEMaxLength(owner);
    addUniqueFacet(maxLength);
    return maxLength;
  }

  public XsEMaxLength getMaxLength() {
    return (XsEMaxLength) getFacetByName("maxLength");
  }

  public XsEWhiteSpace createWhiteSpace() {
    XsEWhiteSpace whiteSpace = owner.getObjectFactory().newXsEWhiteSpace(owner);
    addUniqueFacet(whiteSpace);
    return whiteSpace;
  }

  public XsEWhiteSpace getWhiteSpace() {
    return (XsEWhiteSpace) getFacetByName("whiteSpace");
  }

  public XsEPattern createPattern() {
    XsEPattern pattern = owner.getObjectFactory().newXsEPattern(owner);
    addFacet(pattern);
    return pattern;
  }

  public XsEPattern[] getPatterns() {
    if (facets == null) {
      return new XsEPattern[0];
    }
    List result = new ArrayList();
    for (int i = 0;  i < facets.size();  i++) {
      XsTFacetBase facet = (XsTFacetBase) facets.get(i);
      if ("pattern".equals(facet.getFacetName())) {
        result.add(facet);
      }
    }
    return (XsEPattern[]) result.toArray(new XsEPattern[result.size()]);
  }

  public XsEEnumeration createEnumeration() {
    XsEEnumeration enumeration = owner.getObjectFactory().newXsEEnumeration(owner);
    addFacet(enumeration);
    return enumeration;
  }

  public XsEEnumeration[] getEnumerations() {
    if (facets == null) {
      return new XsEEnumeration[0];
    }
    List result = new ArrayList();
    for (int i = 0;  i < facets.size();  i++) {
      XsTFacetBase facet = (XsTFacetBase) facets.get(i);
      if ("enumeration".equals(facet.getFacetName())) {
        result.add(facet);
      }
    }
    return (XsEEnumeration[]) result.toArray(new XsEEnumeration[result.size()]);
  }

  public boolean hasFacets() {
    return facets != null  &&  facets.size() > 0;
  }

  public XsTFacetBase[] getFacets() {
    if (facets == null) {
      return new XsEEnumeration[0];
    }
    return (XsTFacetBase[]) facets.toArray(new XsTFacetBase[facets.size()]);
  }
}
