/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml.impl;

import org.apache.ws.jaxme.xs.xml.*;


/** <p>Implementation of a named top-level group, with the following
 * specification:
 * <pre>
 *  &lt;xs:complexType name="namedGroup"&gt;
 *    &lt;xs:annotation&gt;
 *      &lt;xs:documentation&gt;
 *        Should derive this from realGroup, but too complicated for now
 *      &lt;/xs:documentation&gt;
 *    &lt;/xs:annotation&gt;
 *    &lt;xs:sequence&gt;
 *      &lt;xs:element ref="xs:annotation" minOccurs="0"/&gt;
 *      &lt;xs:choice minOccurs="1" maxOccurs="1"&gt;
 *        &lt;xs:element name="all"&gt;
 *          &lt;xs:complexType&gt;
 *            &lt;xs:complexContent&gt;
 *              &lt;xs:restriction base="xs:all"&gt;
 *                &lt;xs:group ref="xs:allModel"/&gt;
 *                &lt;xs:attribute name="minOccurs" use="prohibited"/&gt;
 *                &lt;xs:attribute name="maxOccurs" use="prohibited"/&gt;
 *              &lt;/xs:restriction&gt;
 *            &lt;/xs:complexContent&gt;
 *          &lt;/xs:complexType&gt;
 *        &lt;/xs:element&gt;
 *        &lt;xs:element name="choice" type="xs:simpleExplicitGroup"/&gt;
 *        &lt;xs:element name="sequence" type="xs:simpleExplicitGroup"/&gt;
 *      &lt;/xs:choice&gt;
 *    &lt;/xs:sequence&gt;
 *    &lt;xs:attribute name="name" use="required" type="xs:NCName"/&gt;
 *    &lt;xs:attribute name="ref" use="prohibited"/&gt;
 *    &lt;xs:attribute name="minOccurs" use="prohibited"/&gt;
 *    &lt;xs:attribute name="maxOccurs" use="prohibited"/&gt;
 *  &lt;/xs:complexType&gt;
 * &lt;/pre>&lt;/p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsTNamedGroupImpl extends XsTAnnotatedImpl implements XsTNamedGroup {
  private XsGAllModel all;
  private XsTSimpleExplicitGroup sequence, choice;
  private XsNCName name;

  protected XsTNamedGroupImpl(XsObject pParent) {
    super(pParent);
  }

  public void setName(XsNCName pName) {
    name = pName;
  }

  public XsNCName getName() {
    return name;
  }

  public XsGAllModel createAll() {
    if (all != null) {
      throw new IllegalStateException("Multiple 'all' child elements are forbidden.");
    }
    if (choice != null) {
      throw new IllegalStateException("The 'choice' and 'all' child elements are mutually exclusive.");
    }
    if (sequence != null) {
      throw new IllegalStateException("The 'sequence' and 'all' child elements are mutually exclusive.");
    }
    return all = getObjectFactory().newXsGAllModel(this);
  }

  public XsGAllModel getAll() {
    return all;
  }

  public XsTSimpleExplicitGroup createSequence() {
    if (sequence != null) {
      throw new IllegalStateException("Multiple 'sequence' child elements are forbidden.");
    }
    if (all != null) {
      throw new IllegalStateException("The 'all' and 'sequence' child elements are mutually exclusive.");
    }
    if (choice != null) {
      throw new IllegalStateException("The 'all' and 'sequence' child elements are mutually exclusive.");
    }
    return sequence = getObjectFactory().newXsTSimpleExplicitGroup(this);
  }

  public XsTSimpleExplicitGroup getSequence() {
    return sequence;
  }

  public XsTSimpleExplicitGroup createChoice() {
    if (choice != null) {
      throw new IllegalStateException("Multiple 'choice' elements are forbidden.");
    }
    if (sequence != null) {
      throw new IllegalStateException("The 'sequence' and 'choice' elements are mutually exclusive.");
    }
    if (all != null) {
      throw new IllegalStateException("The 'all' and 'choice' elements are mutually exclusive.");
    }
    return choice = getObjectFactory().newXsTSimpleExplicitGroup(this);
  }

  public XsTSimpleExplicitGroup getChoice() {
    return choice;
  }

  public void validate() {
    XsGAllModel myAll = getAll();
    if (myAll == null) {
      XsTSimpleExplicitGroup mySequence = getSequence();
      if (mySequence == null) {
        XsTSimpleExplicitGroup myChoice = getChoice();
        if (myChoice == null) {
          throw new NullPointerException("Neither of the 'all', 'choice', or 'sequence' elements are given.");
        } else {
        }
      } else {
      }
    } else {
    }
  }
}
