/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.xs.xml.impl;

import org.apache.ws.jaxme.xs.parser.impl.LocSAXException;
import org.apache.ws.jaxme.xs.xml.*;
import org.xml.sax.SAXException;


/** <p>Implementation of the following specification for
 * <code>xs:simpleType</code>:
 * <pre>
 *  &lt;xs:complexType name="simpleType" abstract="true"&gt;
 *    &lt;xs:complexContent&gt;
 *      &lt;xs:extension base="xs:annotated"&gt;
 *        &lt;xs:group ref="xs:simpleDerivation"/&gt;
 *        &lt;xs:attribute name="final" type="xs:simpleDerivationSet"/&gt;
 *        &lt;xs:attribute name="name" type="xs:NCName"&gt;
 *          &lt;xs:annotation&gt;
 *            &lt;xs:documentation&gt;
 *              Can be restricted to required or forbidden
 *            &lt;/xs:documentation&gt;
 *          &lt;/xs:annotation&gt;
 *        &lt;/xs:attribute&gt;
 *      &lt;/xs:extension&gt;
 *    &lt;/xs:complexContent&gt;
 *  &lt;/xs:complexType&gt;
 *
 *  &lt;xs:group name="simpleDerivation"&gt;
 *    &lt;xs:choice&gt;
 *      &lt;xs:element ref="xs:restriction"/&gt;
 *      &lt;xs:element ref="xs:list"/&gt;
 *      &lt;xs:element ref="xs:union"/&gt;
 *    &lt;/xs:choice&gt;
 *  &lt;/xs:group&gt;
 * </pre></p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class XsTSimpleTypeImpl extends XsTAnnotatedImpl implements XsTSimpleType {
  private XsERestriction restriction;
  private XsEList list;
  private XsEUnion union;
  private XsSimpleDerivationSet finalSet;
  private XsNCName name;

  protected XsTSimpleTypeImpl(XsObject pParent) {
    super(pParent);
  }

  public XsERestriction createRestriction() throws SAXException {
    if (restriction != null  ||  list != null  ||  union != null) {
      throw new LocSAXException("Only one 'restriction', 'list', or 'union' child element is allowed.", getLocator());
    }
    return restriction = getObjectFactory().newXsERestriction(this);
  }

  public XsERestriction getRestriction() {
    return restriction;
  }

  public XsEList createList() throws SAXException {
    if (restriction != null  ||  list != null  ||  union != null) {
      throw new LocSAXException("Only one 'restriction', 'list', or 'union' child element is allowed.", getLocator());
    }
    return list = getObjectFactory().newXsEList(this);
  }

  public XsEList getList() {
    return list;
  }

  public XsEUnion createUnion() throws SAXException {
    if (restriction != null  ||  list != null  ||  union != null) {
      throw new LocSAXException("Only one 'restriction', 'list', or 'union' child element is allowed.", getLocator());
    }
    return union = getObjectFactory().newXsEUnion(this);
  }

  public XsEUnion getUnion() {
    return union;
  }

  public void setFinal(XsSimpleDerivationSet pSet) throws SAXException {
    finalSet = pSet;
  }

  public XsSimpleDerivationSet getFinal() {
    return finalSet;
  }

  public void setName(XsNCName pName) throws SAXException {
    name = pName;
  }

  public XsNCName getName() {
    return name;
  }

  public void validate() throws SAXException {
    super.validate();
    if (restriction == null  &&  list == null  &&  union == null) {
      throw new LocSAXException("Expected either of 'restriction', 'list' or 'union' child element.", getLocator());
    }
  }
}
